# -*- coding: utf-8 -*-
import collections
import math
import numbers
from abjad.tools import mathtools
from abjad.tools import stringtools
from abjad.tools.pitchtools.Pitch import Pitch


class NamedPitch(Pitch):
    '''Named pitch.

    ..  container:: example

        Initializes from pitch name:

        ::

            >>> pitch = NamedPitch("cs''")
            >>> show(pitch) # doctest: +SKIP

    ..  container:: example

        Initializes from pitch-class / octave string:

        ::

            >>> pitch = NamedPitch('C#5')
            >>> show(pitch) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_alteration_in_semitones',
        '_diatonic_pitch_class_number',
        '_octave_number',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.tools import pitchtools
        if (
            arguments and
            isinstance(arguments[0], collections.Iterable) and
            not stringtools.is_string(arguments[0]) and
            len(arguments) == 1
            ):
            arguments = arguments[0]
        if len(arguments) == 1:
            if isinstance(arguments[0], (int, float)):
                argument = mathtools.integer_equivalent_number_to_integer(
                    float(arguments[0]))
                self._initialize_by_pitch_number(argument)
            elif isinstance(arguments[0], type(self)):
                self._initialize_by_named_pitch(*arguments)
            elif isinstance(arguments[0], pitchtools.NumberedPitch):
                self._initialize_by_pitch_number(
                    arguments[0].pitch_number)
            elif isinstance(arguments[0], pitchtools.PitchClass):
                self._initialize_by_named_pitch_class_and_octave_number(
                    pitchtools.NamedPitchClass(arguments[0]), 4)
            elif hasattr(arguments[0], 'named_pitch'):
                self._initialize_by_named_pitch(arguments[0].named_pitch)
            elif self.is_pitch_class_octave_number_string(arguments[0]):
                self._initialize_by_pitch_class_octave_number_string(*arguments)
            elif isinstance(arguments[0], str):
                self._initialize_by_pitch_name(*arguments)
            else:
                message = 'can not initialize {} from {!r}.'
                message = message.format(type(self).__name__, arguments)
                raise ValueError(message)
        elif len(arguments) == 2:
            if isinstance(arguments[0], str):
                self._initialize_by_pitch_class_name_and_octave_number(
                    *arguments)
            elif isinstance(arguments[0], pitchtools.NamedPitchClass):
                self._initialize_by_named_pitch_class_and_octave_number(
                    *arguments)
            elif isinstance(arguments[0], (int, float)):
                if isinstance(arguments[1], str):
                    self._initialize_by_pitch_number_and_diatonic_pitch_class_name(
                        *arguments)
                elif isinstance(arguments[1], (int, float)):
                    self._initialize_by_pitch_number_and_octave_number(
                        *arguments)
                elif isinstance(arguments[1], pitchtools.NamedPitchClass):
                    self._initialize_by_pitch_number_and_named_pitch_class(
                        *arguments)
                else:
                    message = 'can not initialize {}: {!r}.'
                    message = message.format(type(self).__name__, arguments)
                    raise TypeError(message)
            else:
                message = 'can not initialize {}: {!r}.'
                message = message.format(type(self).__name__, arguments)
                raise ValueError(message)
        elif len(arguments) == 0:
            self._initialize_by_pitch_class_name_and_octave_number('c', 4)
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arguments)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __add__(self, interval):
        r'''Adds named pitch to `interval`.

        ..  container:: example

            Adds an ascending major second to C#5:

            ::

                >>> NamedPitch("cs''") + NamedInterval('+M2')
                NamedPitch("ds''")

        ..  container:: example

            Adds a descending major second to C#5:

            ::

                >>> NamedPitch("cs''") + NamedInterval('-M2')
                NamedPitch("b'")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NamedInterval(interval)
        pitch = interval.transpose(self)
        return pitch

    def __copy__(self, *arguments):
        r'''Copies named pitch.

        ..  container:: example

            Copies C#5:

            ::

                >>> import copy
                >>> copy.copy(NamedPitch("cs''"))
                NamedPitch("cs''")

        ..  container:: example

            Copies Db5:

            ::

                >>> copy.copy(NamedPitch("df''"))
                NamedPitch("df''")

        Returns new named pitch.
        '''
        return type(self)(self)

    def __eq__(self, argument):
        r'''Is true when `argument` is a named pitch equal to this named pitch.
        Otherwise false.

        ..  container:: example

            C#5 equals C#5:

            ::

                >>> NamedPitch('C#5') == NamedPitch("cs''")
                True

        ..  container:: example

            C#5 does not equal Db5:

            ::

                >>> NamedPitch('C#5') == NamedPitch('Db5')
                False

        Returns true or false.
        '''
        try:
            argument = type(self)(argument)
            if str(self) == str(argument):
                return True
            return False
        except (TypeError, ValueError):
            return False

    def __float__(self):
        r'''Changes named pitch to float.

        ..  container:: example

            Changes C#5 to float:

            ::

                >>> float(NamedPitch('C#5'))
                13.0

        ..  container:: example

            Changes Ctqs5 to float:

            ::

                >>> float(NamedPitch('C#+5'))
                13.5

        Returns float.
        '''
        return float(self.pitch_number)

    def __ge__(self, argument):
        r'''Is true when named pitch is greater than or equal to `argument`.
        Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            return self.diatonic_pitch_number > argument.diatonic_pitch_number or \
                (self.diatonic_pitch_number == argument.diatonic_pitch_number and
                self.alteration_in_semitones >= argument.alteration_in_semitones)
        elif isinstance(argument, pitchtools.PitchRange):
            return self >= argument.stop_pitch
        else:
            try:
                argument = type(self)(argument)
                return self.__ge__(argument)
            except (TypeError, ValueError):
                pass
        return False

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.pitch_name,)

    def __gt__(self, argument):
        r'''Is true when named pitch is greater than `argument`. Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            return (self.diatonic_pitch_number > argument.diatonic_pitch_number or
                (self.diatonic_pitch_number == argument.diatonic_pitch_number and
                self.alteration_in_semitones > argument.alteration_in_semitones))
        elif isinstance(argument, pitchtools.PitchRange):
            return self > argument.stop_pitch
        else:
            try:
                argument = type(self)(argument)
                return self.__gt__(argument)
            except (TypeError, ValueError):
                pass
        return False

    def __hash__(self):
        r'''Required to be explicitly redefined on Python 3 if
        __eq__ changes.

        Returns integer.
        '''
        return super(NamedPitch, self).__hash__()

    def __int__(self):
        r'''Changes named pitch to integer.

        ..  container:: example

            Changes C#5 to integer:

            ::

                >>> int(NamedPitch('C#5'))
                13

        ..  container:: example

            Changes Db5 to integer:

            ::

                >>> int(NamedPitch('Db5'))
                13

        Returns integer.
        '''
        if not mathtools.is_integer_equivalent_number(self.pitch_number):
            raise TypeError
        return int(self.pitch_number)

    def __le__(self, argument):
        r'''Is true when named pitch is less than or equal to `argument`. Otherwise
        false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            if not self.diatonic_pitch_number == argument.diatonic_pitch_number:
                return self.diatonic_pitch_number <= argument.diatonic_pitch_number
            if not self.alteration_in_semitones == argument.alteration_in_semitones:
                return self.alteration_in_semitones <= \
                    argument.alteration_in_semitones
            return True
        elif isinstance(argument, pitchtools.PitchRange):
            return self <= argument.start_pitch
        else:
            try:
                argument = type(self)(argument)
                return self.__le__(argument)
            except (TypeError, ValueError):
                pass
        return False

    def __lt__(self, argument):
        r'''Is true when named pitch is less than `argument`. Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            return (self.diatonic_pitch_number < argument.diatonic_pitch_number or
                (self.diatonic_pitch_number == argument.diatonic_pitch_number and
                self.alteration_in_semitones < argument.alteration_in_semitones))
        elif isinstance(argument, pitchtools.PitchRange):
            return self < argument.start_pitch
        elif argument is None:
            return True
        else:
            try:
                argument = type(self)(argument)
                return self.__lt__(argument)
            except (TypeError, ValueError):
                pass
        return False

    def __ne__(self, argument):
        r'''Is true when named pitch does not equal `argument`. Otherwise false.

        ..  container:: example

            C#5 is not equal to D#5:

            ::

                >>> NamedPitch("cs''") != NamedPitch("ds''")
                True

        ..  container:: example

            C#5 is equal to C#5:

            ::

                >>> NamedPitch("cs''") != NamedPitch("cs''")
                False

        Returns true or false.
        '''
        return not self == argument

    def __str__(self):
        r'''Gets string representation of named pitch.

        ..  container:: example

            Gets string representation of C#5:

            ::

                >>> str(NamedPitch("cs''"))
                "cs''"

        ..  container:: example

            Gets string representation of Db5:

            ::

                >>> str(NamedPitch("df''"))
                "df''"

        Returns string.
        '''
        return self.pitch_name

    def __sub__(self, argument):
        r'''Subtracts `argument` from named pitch.

        ..  container:: example

            Subtracts B4 from C#5:

            ::

                >>> NamedPitch("cs''") - NamedPitch("b'")
                NamedInterval('-M2')

        ..  container:: example

            Subtracts F#5 from C#5:

            ::

                >>> NamedPitch("cs''") - NamedPitch("fs''")
                NamedInterval('+P4')

        Returns named interval.
        '''
        from abjad.tools import pitchtools
        if isinstance(argument, type(self)):
            return pitchtools.NamedInterval.from_pitch_carriers(self, argument)
        interval = pitchtools.NamedInterval(argument)
        interval = -interval
        pitch = interval.transpose(self)
        return pitch

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _initialize_by_named_pitch(self, named_pitch):
        self._alteration_in_semitones = named_pitch._alteration_in_semitones
        self._diatonic_pitch_class_number = \
            named_pitch.diatonic_pitch_class_number
        self._octave_number = named_pitch.octave.number

    def _initialize_by_named_pitch_class_and_octave_number(
        self, named_pitch_class, octave_number):
        self._alteration_in_semitones = \
            named_pitch_class._alteration_in_semitones
        self._diatonic_pitch_class_number = \
            named_pitch_class._diatonic_pitch_class_number
        self._octave_number = int(octave_number)

    def _initialize_by_pitch_class_name_and_octave_number(
        self, pitch_class_name, octave_number):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_class_name)
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_class_name_octave_number_pair(self, pair):
        pitch_class_name, octave_number = pair
        self._initialize_by_pitch_class_name_and_octave_number(
            pitch_class_name, octave_number)

    def _initialize_by_pitch_class_octave_number_string(
        self, pitch_class_octave_number_string):
        from abjad.tools import pitchtools
        group_dict = self._pitch_class_octave_number_regex.match(
            pitch_class_octave_number_string).groupdict()
        named_pitch_class = pitchtools.NamedPitchClass(
            pitch_class_octave_number_string)
        octave_number = int(group_dict['octave_number'])
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_name(self, pitch_string):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_string)
        octave = pitchtools.Octave.from_pitch_name(pitch_string)
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave.number)

    def _initialize_by_pitch_number(self, pitch_number):
        from abjad.tools import pitchtools
        named_pitch_class = pitchtools.NamedPitchClass(pitch_number)
        octave_number = pitch_number // 12 + 4
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_number_and_diatonic_pitch_class_name(
        self, pitch_number, diatonic_pitch_class_name):
        from abjad.tools import pitchtools
        accidental, octave_number = self._spell_pitch_number(
            pitch_number,
            diatonic_pitch_class_name,
            )
        pitch_class_name = diatonic_pitch_class_name + \
            accidental.abbreviation
        named_pitch_class = pitchtools.NamedPitchClass(pitch_class_name)
        self._initialize_by_named_pitch_class_and_octave_number(
            named_pitch_class, octave_number)

    def _initialize_by_pitch_number_and_named_pitch_class(
        self, pitch_number, named_pitch_class):
        diatonic_pitch_class_name = named_pitch_class.diatonic_pitch_class_name
        self._initialize_by_pitch_number_and_diatonic_pitch_class_name(
            pitch_number, diatonic_pitch_class_name)

    def _initialize_by_pitch_number_and_octave_number(
        self, pitch_number, octave_number):
        pitch_class_name = type(self)(pitch_number).pitch_class_name
        octave_number = int(octave_number)
        self._initialize_by_pitch_class_name_and_octave_number(
            pitch_class_name, octave_number)

    @staticmethod
    def _spell_pitch_number(pitch_number, diatonic_pitch_class_name):
        from abjad.tools import pitchtools
        # check input
        if not isinstance(pitch_number, (int, float)):
            raise TypeError
        if not isinstance(diatonic_pitch_class_name, str):
            raise TypeError
        if diatonic_pitch_class_name not in ['c', 'd', 'e', 'f', 'g', 'a', 'b']:
            raise ValueError
        # find accidental semitones
        pc = pitchtools.PitchClass._diatonic_pitch_class_name_to_pitch_class_number[
            diatonic_pitch_class_name]
        nearest_neighbor = NamedPitch._to_nearest_octave(pitch_number, pc)
        semitones = pitch_number - nearest_neighbor
        # find accidental alphabetic string
        abbreviation = pitchtools.Accidental._semitones_to_abbreviation[
            semitones]
        accidental = pitchtools.Accidental(abbreviation)
        # find octave
        octave_number = int(math.floor((pitch_number - semitones) / 12)) + 4
        # return accidental and octave
        return accidental, octave_number

    @staticmethod
    def _to_nearest_octave(pitch_number, pitch_class_number):
        target_pc = pitch_number % 12
        down = (target_pc - pitch_class_number) % 12
        up = (pitch_class_number - target_pc) % 12
        if up < down:
            return pitch_number + up
        else:
            return pitch_number - down

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Gets accidental of named pitch.

        ..  container:: example

            Gets accidental of C#5:

            ::

                >>> NamedPitch("cs''").accidental
                Accidental('s')

        ..  container:: example

            Gets accidental of C5:

            ::

                >>> NamedPitch("c''").accidental
                Accidental('')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self._alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Gets alteration of named pitch in semitones.

        ..  container:: example

            Gets alteration of C#5 in semitones:

            ::

                >>> NamedPitch("cs''").alteration_in_semitones
                1

        ..  container:: example

            Gets alteration of Ctqs5 in semitones:

            ::

                >>> NamedPitch("ctqs''").alteration_in_semitones
                1.5

        Returns integer or float.
        '''
        return self._alteration_in_semitones

    @property
    def diatonic_pitch_class_name(self):
        r'''Gets diatonic pitch-class name of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets diatonic pitch-class name of C#5:

            ::

                >>> NamedPitch("cs''").diatonic_pitch_class_name
                'c'

        ..  container:: example

            Gets diatonic pitch-class names of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> for note in staff:
                ...     name = note.written_pitch.diatonic_pitch_class_name
                ...     markup = Markup(name)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { g }
                    a16 - \markup { a }
                    b16 - \markup { b }
                    c'16 - \markup { c }
                    d'16 - \markup { d }
                    e'16 - \markup { e }
                    f'16 - \markup { f }
                    g'16 - \markup { g }
                    a'16 - \markup { a }
                    b'16 - \markup { b }
                    c''16 - \markup { c }
                    d''16 - \markup { d }
                    e''16 - \markup { e }
                    f''16 - \markup { f }
                    g''16 - \markup { g }
                    a''16 - \markup { a }
                }

        Returns string.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        return class_._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
            self._diatonic_pitch_class_number]

    @property
    def diatonic_pitch_class_number(self):
        r'''Gets diatonic pitch-class number of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets diatonic pitch-class number of C#5:

            ::

                >>> NamedPitch("cs''").diatonic_pitch_class_number
                0

        ..  container:: example

            Gets diatonic pitch-class numbers of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> for note in staff:
                ...     number = note.written_pitch.diatonic_pitch_class_number
                ...     markup = Markup(number)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { 4 }
                    a16 - \markup { 5 }
                    b16 - \markup { 6 }
                    c'16 - \markup { 0 }
                    d'16 - \markup { 1 }
                    e'16 - \markup { 2 }
                    f'16 - \markup { 3 }
                    g'16 - \markup { 4 }
                    a'16 - \markup { 5 }
                    b'16 - \markup { 6 }
                    c''16 - \markup { 0 }
                    d''16 - \markup { 1 }
                    e''16 - \markup { 2 }
                    f''16 - \markup { 3 }
                    g''16 - \markup { 4 }
                    a''16 - \markup { 5 }
                }

        Returns integer.
        '''
        return self._diatonic_pitch_class_number

    @property
    def diatonic_pitch_name(self):
        r'''Gets diatonic pitch name of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets diatonic pitch name of C#5:

            ::

                >>> NamedPitch("cs''").diatonic_pitch_name
                "c''"

        ..  container:: example

            Gets diatonic pitch names of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> for note in staff:
                ...     name = note.written_pitch.diatonic_pitch_name
                ...     markup = Markup(name)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { g }
                    a16 - \markup { a }
                    b16 - \markup { b }
                    c'16 - \markup { c' }
                    d'16 - \markup { d' }
                    e'16 - \markup { e' }
                    f'16 - \markup { f' }
                    g'16 - \markup { g' }
                    a'16 - \markup { a' }
                    b'16 - \markup { b' }
                    c''16 - \markup { c'' }
                    d''16 - \markup { d'' }
                    e''16 - \markup { e'' }
                    f''16 - \markup { f'' }
                    g''16 - \markup { g'' }
                    a''16 - \markup { a'' }
                }

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            self.octave.tick_string,
            )

    @property
    def diatonic_pitch_number(self):
        r'''Gets diatonic pitch number of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets diatonic pitch number of C#5:

            ::

                >>> NamedPitch("cs''").diatonic_pitch_number
                7

        ..  container:: example

            Gets diatonic pitch numbers of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> for note in staff:
                ...     number = note.written_pitch.diatonic_pitch_number
                ...     markup = Markup(number)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { -3 }
                    a16 - \markup { -2 }
                    b16 - \markup { -1 }
                    c'16 - \markup { 0 }
                    d'16 - \markup { 1 }
                    e'16 - \markup { 2 }
                    f'16 - \markup { 3 }
                    g'16 - \markup { 4 }
                    a'16 - \markup { 5 }
                    b'16 - \markup { 6 }
                    c''16 - \markup { 7 }
                    d''16 - \markup { 8 }
                    e''16 - \markup { 9 }
                    f''16 - \markup { 10 }
                    g''16 - \markup { 11 }
                    a''16 - \markup { 12 }
                }

        Returns integer.
        '''
        diatonic_pitch_number = 7 * (self.octave.number - 4)
        diatonic_pitch_number += self.diatonic_pitch_class_number
        return diatonic_pitch_number

    @property
    def name(self):
        r'''Gets pitch name.

        ..  container:: example

            ::

                >>> NamedPitch("cs''").name
                "cs''"

        Returns string.
        '''
        return self.pitch_name

    @property
    def named_pitch(self):
        r'''Gets new named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets new named pitch from C#5:

            ::

                >>> NamedPitch("cs''").named_pitch
                NamedPitch("cs''")

        ..  container:: example

            Gets new named pitch from Db5:

            ::

                >>> NamedPitch("df''").named_pitch
                NamedPitch("df''")

        Returns new named pitch.
        '''
        return type(self)(self)

    @property
    def named_pitch_class(self):
        r'''Gets named pitch-class of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets named pitch-class of C#5:

            ::

                >>> NamedPitch("cs''").named_pitch_class
                NamedPitchClass('cs')

        ..  container:: example

            Gets named pitch-class of Db5:

            ::

                >>> NamedPitch("df''").named_pitch_class
                NamedPitchClass('df')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def number(self):
        r'''Gets pitch number.

        ..  container:: ex

            Gets pitch number of C#5:

            ::

                >>> NamedPitch("cs''").number
                13


        Returns number.
        '''
        return self.numbered_pitch.number

    @property
    def numbered_pitch(self):
        r'''Gets numbered pitch corresponding to named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets numbered pitch corresponding to C#5:

            ::

                >>> NamedPitch("cs''").numbered_pitch
                NumberedPitch(13)

        ..  container:: example

            Gets numbered pitch corresponding to Db5:

            ::

                >>> NamedPitch("df''").numbered_pitch
                NumberedPitch(13)

        Returns numbered pitch.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch(self)

    @property
    def numbered_pitch_class(self):
        r'''Gets numbered pitch-class corresponding to named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets numbered pitch-class corresponding to C#5:

            ::

                >>> NamedPitch("cs''").numbered_pitch_class
                NumberedPitchClass(1)

        ..  container:: example

            Gets numbered pitch-class corresponding to Db5:

            ::

                >>> NamedPitch("df''").numbered_pitch_class
                NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def octave(self):
        r'''Gets octave of named pitch.

        ..  container:: example

            Gets octave of C#5:

            ::

                >>> NamedPitch("cs''").octave
                Octave(5)

        ..  container:: example

            Gets octave of Db5:

            ::

                >>> NamedPitch("df''").octave
                Octave(5)

        Returns octave.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Octave(self._octave_number)

    @property
    def pitch_class(self):
        r'''Gets pitch-class.

        ..  container:: example

            ::

                >>> NamedPitch("cs''").pitch_class
                NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def pitch_class_name(self):
        r'''Gets pitch-class name of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets pitch-class name of C#5:

            ::

                >>> NamedPitch("cs''").pitch_class_name
                'cs'

        ..  container:: example

            Gets pitch-class name of Db5:

            ::

                >>> NamedPitch("df''").pitch_class_name
                'df'

        Returns string.
        '''
        from abjad.tools import pitchtools
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            pitchtools.Accidental(self._alteration_in_semitones).abbreviation
            )

    @property
    def pitch_class_number(self):
        r'''Gets pitch-class number of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets pitch-class number of C#5:

            ::

                >>> NamedPitch("cs''").pitch_class_number
                1

        ..  container:: example

            Gets pitch-class number of Ctqs5:

            ::

                >>> NamedPitch("ctqs''").pitch_class_number
                1.5

        Returns integer or float.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        return (class_._diatonic_pitch_class_number_to_pitch_class_number[
            self._diatonic_pitch_class_number] +
            self._alteration_in_semitones) % 12

    @property
    def pitch_class_octave_label(self):
        r'''Gets pitch-class / octave label of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets pitch-class / octave label of C#5:

            ::

                >>> NamedPitch("cs''").pitch_class_octave_label
                'C#5'

        ..  container:: example

            Gets pitch-class / octave label of Ctqs5:

            ::

                >>> NamedPitch("ctqs''").pitch_class_octave_label
                'C#+5'

        Returns string.
        '''
        return '{}{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            self.octave.number,
            )

    @property
    def pitch_name(self):
        r'''Gets pitch name of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets pitch name of C#5:

            ::

                >>> NamedPitch("cs''").pitch_name
                "cs''"

        ..  container:: example

            Gets pitch name of Ctqs5:

            ::

                >>> NamedPitch("ctqs''").pitch_name
                "ctqs''"

        Returns string.
        '''
        return '{}{}'.format(
            self.pitch_class_name,
            self.octave.tick_string,
            )

    @property
    def pitch_number(self):
        r'''Gets pitch number of named pitch.

        ..  note:: Deprecated.

        ..  container:: example

            Gets pitch number of C#5:

            ::

                >>> NamedPitch("cs''").pitch_number
                13

        ..  container:: example

            Gets pitch number of Cbb5:

            ::

                >>> NamedPitch("cff''").pitch_number
                10

        Returns integer or float.
        '''
        from abjad.tools import pitchtools
        pitch_class_number = pitchtools.PitchClass._diatonic_pitch_class_number_to_pitch_class_number[
            self.diatonic_pitch_class_number]
        pitch_number = pitch_class_number + 12 * (self.octave.number - 4)
        pitch_number += self.alteration_in_semitones
        return pitch_number

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        '''Applies `accidental` to named pitch.

        ..  container:: example

            Applies sharp to C#5:

            ::

                >>> NamedPitch("cs''").apply_accidental('s')
                NamedPitch("css''")

        ..  container:: example

            Applies sharp to Db5:

            ::

                >>> NamedPitch("df''").apply_accidental('s')
                NamedPitch("d''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        accidental = pitchtools.Accidental(accidental)
        new_accidental = self.accidental + accidental
        new_name = self.diatonic_pitch_class_name
        new_name += new_accidental.abbreviation
        return type(self)(new_name, self.octave.number)

    @staticmethod
    def from_pitch_carrier(pitch_carrier):
        r'''Initializes named pitch from `pitch_carrier`.

        ..  container:: example

            Initializes named pitch from named pitch:

            ::

                >>> pitch = NamedPitch('df', 5)
                >>> NamedPitch.from_pitch_carrier(pitch)
                NamedPitch("df''")

        ..  container:: example

            Initializes named pitch from note:

            ::

                >>> note = Note("df''4")
                >>> NamedPitch.from_pitch_carrier(note)
                NamedPitch("df''")

        ..  container:: example

            Initializes named pitch from note-head:

            ::

                >>> note = Note("df''4")
                >>> NamedPitch.from_pitch_carrier(note.note_head)
                NamedPitch("df''")

        ..  container:: example

            Initializes named pitch from chord:

            ::

                >>> chord = Chord("<df''>4")
                >>> NamedPitch.from_pitch_carrier(chord)
                NamedPitch("df''")

        ..  container:: example

            Initializes named pitch from integer:

            ::

                >>> NamedPitch.from_pitch_carrier(13)
                NamedPitch("cs''")

        ..  container:: example

            Initializes named pitch from numbered pitch-class:

            ::

                >>> pitch_class = NumberedPitchClass(7)
                >>> NamedPitch.from_pitch_carrier(pitch_class)
                NamedPitch("g'")

        Raises value error when `pitch_carrier` carries no pitch.

        Raises value error when `pitch_carrier` carries more than one pitch.

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        if isinstance(pitch_carrier, pitchtools.NamedPitch):
            return pitch_carrier
        elif isinstance(pitch_carrier, pitchtools.NumberedPitch):
            return pitchtools.NamedPitch(pitch_carrier)
        elif isinstance(pitch_carrier, numbers.Number):
            return pitchtools.NamedPitch(pitch_carrier)
        elif isinstance(pitch_carrier, scoretools.Note):
            pitch = pitch_carrier.written_pitch
            if pitch is not None:
                return NamedPitch.from_pitch_carrier(pitch)
            else:
                message = 'no pitch found on {!r}.'
                message = message.format(pitch_carrier)
                raise ValueError(message)
        elif isinstance(pitch_carrier, scoretools.NoteHead):
            pitch = pitch_carrier.written_pitch
            if pitch is not None:
                return NamedPitch.from_pitch_carrier(pitch)
            else:
                message = 'no pitch found on {!r}.'
                message = message.format(pitch_carrier)
                raise ValueError(message)
        elif isinstance(pitch_carrier, scoretools.Chord):
            pitches = pitch_carrier.written_pitches
            if len(pitches) == 0:
                message = 'no pitch found on {!r}.'
                message = message.format(pitch_carrier)
                raise ValueError(message)
            elif len(pitches) == 1:
                return NamedPitch.from_pitch_carrier(pitches[0])
            else:
                message = 'multiple pitches found on {!r}.'
                message = message.format(pitch_carrier)
                raise ValueError(message)
        elif isinstance(pitch_carrier, pitchtools.NumberedPitchClass):
            named_pitch_class = pitch_carrier.named_pitch_class
            named_pitch = pitchtools.NamedPitch(named_pitch_class)
            return named_pitch
        else:
            message = 'pitch carrier {!r} must be'
            message += ' pitch, note, note-head or chord.'
            message = message.format(pitch_carrier)
            raise TypeError(message)

    @staticmethod
    def from_staff_position(staff_position, clef=None):
        r'''Initializes named pitch from `staff_position` and optional `clef`.

        ..  container:: example

            Initializes notes from absolute staff positions:

            ::

                >>> for n in range(-6, 6):
                ...     staff_position = pitchtools.StaffPosition(n)
                ...     pitch = NamedPitch.from_staff_position(staff_position)
                ...     message = '{!s}\t{}'.format(staff_position, pitch)
                ...     print(message)
                StaffPosition(-6)	d
                StaffPosition(-5)	e
                StaffPosition(-4)	f
                StaffPosition(-3)	g
                StaffPosition(-2)	a
                StaffPosition(-1)	b
                StaffPosition(0)	c'
                StaffPosition(1)	d'
                StaffPosition(2)	e'
                StaffPosition(3)	f'
                StaffPosition(4)	g'
                StaffPosition(5)	a'

        ..  container:: example

            Initializes notes inside treble staff from staff positions:

            ::

                >>> clef = Clef('treble')
                >>> for n in range(-6, 6):
                ...     staff_position = pitchtools.StaffPosition(n)
                ...     pitch = NamedPitch.from_staff_position(
                ...         staff_position,
                ...         clef=clef,
                ...         )
                ...     message = '{!s}\t{}'.format(staff_position, pitch)
                ...     print(message)
                StaffPosition(-6)	c'
                StaffPosition(-5)	d'
                StaffPosition(-4)	e'
                StaffPosition(-3)	f'
                StaffPosition(-2)	g'
                StaffPosition(-1)	a'
                StaffPosition(0)	b'
                StaffPosition(1)	c''
                StaffPosition(2)	d''
                StaffPosition(3)	e''
                StaffPosition(4)	f''
                StaffPosition(5)	g''

        ..  container:: example

            Initializes notes inside bass staff from staff positions:

            ::

                >>> clef = Clef('bass')
                >>> for n in range(-6, 6):
                ...     staff_position = pitchtools.StaffPosition(n)
                ...     pitch = NamedPitch.from_staff_position(
                ...         staff_position,
                ...         clef=clef,
                ...         )
                ...     message = '{!s}\t{}'.format(staff_position, pitch)
                ...     print(message)
                StaffPosition(-6)	e,
                StaffPosition(-5)	f,
                StaffPosition(-4)	g,
                StaffPosition(-3)	a,
                StaffPosition(-2)	b,
                StaffPosition(-1)	c
                StaffPosition(0)	d
                StaffPosition(1)	e
                StaffPosition(2)	f
                StaffPosition(3)	g
                StaffPosition(4)	a
                StaffPosition(5)	b

        ..  container:: example

            Initializes notes inside alto staff from staff positions:

            ::

                >>> clef = Clef('alto')
                >>> for n in range(-6, 6):
                ...     staff_position = pitchtools.StaffPosition(n)
                ...     pitch = NamedPitch.from_staff_position(
                ...         staff_position,
                ...         clef=clef,
                ...         )
                ...     message = '{!s}\t{}'.format(staff_position, pitch)
                ...     print(message)
                StaffPosition(-6)	d
                StaffPosition(-5)	e
                StaffPosition(-4)	f
                StaffPosition(-3)	g
                StaffPosition(-2)	a
                StaffPosition(-1)	b
                StaffPosition(0)	c'
                StaffPosition(1)	d'
                StaffPosition(2)	e'
                StaffPosition(3)	f'
                StaffPosition(4)	g'
                StaffPosition(5)	a'

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        if not isinstance(staff_position, pitchtools.StaffPosition):
            staff_position = pitchtools.StaffPosition(staff_position)
        if clef is not None:
            offset_staff_position_number = staff_position.number
            offset_staff_position_number -= clef.middle_c_position.number
            offset_staff_position = pitchtools.StaffPosition(
                offset_staff_position_number)
        else:
            offset_staff_position = staff_position
        octave_number = offset_staff_position.number // 7 + 4
        diatonic_pitch_class_number = offset_staff_position.number % 7
        pitch_class_number = pitchtools.PitchClass._diatonic_pitch_class_number_to_pitch_class_number[
            diatonic_pitch_class_number]
        pitch_number = 12 * (octave_number - 4)
        pitch_number += pitch_class_number
        named_pitch = NamedPitch(pitch_number)
        return named_pitch

    def invert(self, axis=None):
        r'''Inverts named pitch around `axis`.

        ..  container:: example

            Inverts pitch around middle C explicitly:

            ::

                >>> NamedPitch("d'").invert("c'")
                NamedPitch('bf')

            ::

                >>> NamedPitch('bf').invert("c'")
                NamedPitch("d'")

        ..  container:: example

            Inverts pitch around middle C implicitly:

            ::

                >>> NamedPitch("d'").invert()
                NamedPitch('bf')

            ::

                >>> NamedPitch('bf').invert()
                NamedPitch("d'")

        ..  container:: example

            Inverts pitch around A3:

            ::

                >>> NamedPitch("d'").invert('a')
                NamedPitch('e')

        Interprets none-valued `axis` equal to middle C.

        Returns new named pitch.
        '''
        try:
            return Pitch.invert(self, axis=axis)
        except:
            return Pitch.invert(type(self)(float(self)), axis=axis)

    def multiply(self, n=1):
        r'''Multiplies pitch-class of named pitch by `n` while maintaining
        octave of named pitch.

        ..  note:: This is wrong. Should not preserve octave.

        ..  container:: example

            Multiplies D2 by 3:

            ::

                >>> NamedPitch('d,').multiply(3)
                NamedPitch('fs,')

        ..  container:: example

            Multiplies D2 by 4:

            ::

                >>> NamedPitch('d,').multiply(4)
                NamedPitch('af,')

        Returns new named pitch.
        '''
        pitch_class_number = (self.pitch_class_number * n) % 12
        octave_floor = (self.octave.number - 4) * 12
        return type(self)(pitch_class_number + octave_floor)

    def respell_with_flats(self):
        r'''Respells named pitch with flats.

        ..  container:: example

            Respells C#5 with flats:

            ::

                >>> NamedPitch("cs''").respell_with_flats()
                NamedPitch("df''")

        ..  container:: example

            Respells Db5 with flats:

            ::

                >>> NamedPitch("df''").respell_with_flats()
                NamedPitch("df''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        octave = pitchtools.Octave.from_pitch_number(
            self.numbered_pitch.pitch_number)
        name = class_._pitch_class_number_to_pitch_class_name_with_flats[
            self.pitch_class_number]
        pitch = type(self)(name, octave.number)
        return pitch

    def respell_with_sharps(self):
        r'''Respells named pitch with sharps.

        ..  container:: example

            Respells Db5 with sharps:

            ::

                >>> NamedPitch("df''").respell_with_sharps()
                NamedPitch("cs''")

        ..  container:: example

            Respells C#5 with sharps:

            ::

                >>> NamedPitch("cs''").respell_with_sharps()
                NamedPitch("cs''")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        class_ = pitchtools.PitchClass
        octave = pitchtools.Octave.from_pitch_number(
            self.numbered_pitch.pitch_number)
        name = class_._pitch_class_number_to_pitch_class_name_with_sharps[
            self.pitch_class_number]
        pitch = type(self)(name, octave.number)
        return pitch

    def to_staff_position(self, clef=None):
        r'''Changes named pitch to staff position with optional `clef`.

        ..  container:: example

            Changes C#5 to absolute staff position:

            ::

                >>> NamedPitch('C#5').to_staff_position()
                StaffPosition(number=7)

        ..  container:: example

            Changes C#5 to treble staff position:

            ::


                >>> NamedPitch('C#5').to_staff_position(clef=Clef('treble'))
                StaffPosition(number=1)

        ..  container:: example

            Changes C#5 to bass staff position:

            ::


                >>> NamedPitch('C#5').to_staff_position(clef=Clef('bass'))
                StaffPosition(number=13)

        ..  container:: example

            Marks up absolute staff position of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> for note in staff:
                ...     staff_position = note.written_pitch.to_staff_position()
                ...     markup = Markup(staff_position.number)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { -3 }
                    a16 - \markup { -2 }
                    b16 - \markup { -1 }
                    c'16 - \markup { 0 }
                    d'16 - \markup { 1 }
                    e'16 - \markup { 2 }
                    f'16 - \markup { 3 }
                    g'16 - \markup { 4 }
                    a'16 - \markup { 5 }
                    b'16 - \markup { 6 }
                    c''16 - \markup { 7 }
                    d''16 - \markup { 8 }
                    e''16 - \markup { 9 }
                    f''16 - \markup { 10 }
                    g''16 - \markup { 11 }
                    a''16 - \markup { 12 }
                }

        ..  container:: example

            Marks up treble staff position of many pitches:

            ::

                >>> staff = Staff("g16 a b c' d' e' f' g' a' b' c'' d'' e'' f'' g'' a''")
                >>> clef = Clef('treble')
                >>> for note in staff:
                ...     staff_position = note.written_pitch.to_staff_position(
                ...         clef=clef
                ...         )
                ...     markup = Markup(staff_position.number)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    g16 - \markup { -9 }
                    a16 - \markup { -8 }
                    b16 - \markup { -7 }
                    c'16 - \markup { -6 }
                    d'16 - \markup { -5 }
                    e'16 - \markup { -4 }
                    f'16 - \markup { -3 }
                    g'16 - \markup { -2 }
                    a'16 - \markup { -1 }
                    b'16 - \markup { 0 }
                    c''16 - \markup { 1 }
                    d''16 - \markup { 2 }
                    e''16 - \markup { 3 }
                    f''16 - \markup { 4 }
                    g''16 - \markup { 5 }
                    a''16 - \markup { 6 }
                }

        ..  container:: example

            Marks up bass staff position of many pitches:

            ::

                >>> staff = Staff("g,16 a, b, c d e f g a b c' d' e' f' g' a'")
                >>> clef = Clef('bass')
                >>> attach(clef, staff)
                >>> for note in staff:
                ...     staff_position = note.written_pitch.to_staff_position(
                ...         clef=clef
                ...         )
                ...     markup = Markup(staff_position.number)
                ...     attach(markup, note)
                ...
                >>> override(staff).text_script.staff_padding = 5
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    \override TextScript.staff-padding = #5
                } {
                    \clef "bass"
                    g,16 - \markup { -4 }
                    a,16 - \markup { -3 }
                    b,16 - \markup { -2 }
                    c16 - \markup { -1 }
                    d16 - \markup { 0 }
                    e16 - \markup { 1 }
                    f16 - \markup { 2 }
                    g16 - \markup { 3 }
                    a16 - \markup { 4 }
                    b16 - \markup { 5 }
                    c'16 - \markup { 6 }
                    d'16 - \markup { 7 }
                    e'16 - \markup { 8 }
                    f'16 - \markup { 9 }
                    g'16 - \markup { 10 }
                    a'16 - \markup { 11 }
                }

        Returns staff position.
        '''
        from abjad.tools import pitchtools
        staff_position_number = self.diatonic_pitch_number
        if clef is not None:
            staff_position_number += clef.middle_c_position.number
        staff_position = pitchtools.StaffPosition(staff_position_number)
        return staff_position

    def transpose(self, n=0):
        r'''Transposes named pitch by index `n`.

        ..  container:: example

            Transposes C4 up a minor second:

            ::

                >>> NamedPitch("c'").transpose(n='m2')
                NamedPitch("df'")

        ..  container:: example

            Transposes C4 down a major second:

            ::

                >>> NamedPitch("c'").transpose(n='-M2')
                NamedPitch('bf')

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        interval = pitchtools.NamedInterval(n)
        pitch = interval.transpose(self)
        return type(self)(pitch)

    def transpose_staff_position(self, staff_positions, interval):
        '''Transposes named pitch by `staff_positions` and `interval`.

        ..  container:: example

            Transposes middle C but leaves at same staff position:

            ::

                >>> pitch = NamedPitch(0)

            ::

                >>> pitch.transpose_staff_position(0, -2)
                NamedPitch("cff'")
                >>> pitch.transpose_staff_position(0, -1.5)
                NamedPitch("ctqf'")
                >>> pitch.transpose_staff_position(0, -1)
                NamedPitch("cf'")
                >>> pitch.transpose_staff_position(0, -0.5)
                NamedPitch("cqf'")
                >>> pitch.transpose_staff_position(0, 0)
                NamedPitch("c'")
                >>> pitch.transpose_staff_position(0, 0.5)
                NamedPitch("cqs'")
                >>> pitch.transpose_staff_position(0, 1)
                NamedPitch("cs'")
                >>> pitch.transpose_staff_position(0, 1.5)
                NamedPitch("ctqs'")

        ..  container:: example

            Transposes middle C and then respells up 1 staff position:

                >>> pitch.transpose_staff_position(1, 0)
                NamedPitch("dff'")
                >>> pitch.transpose_staff_position(1, 0.5)
                NamedPitch("dtqf'")
                >>> pitch.transpose_staff_position(1, 1)
                NamedPitch("df'")
                >>> pitch.transpose_staff_position(1, 1.5)
                NamedPitch("dqf'")
                >>> pitch.transpose_staff_position(1, 2)
                NamedPitch("d'")
                >>> pitch.transpose_staff_position(1, 2.5)
                NamedPitch("dqs'")
                >>> pitch.transpose_staff_position(1, 3)
                NamedPitch("ds'")
                >>> pitch.transpose_staff_position(1, 3.5)
                NamedPitch("dtqs'")
                >>> pitch.transpose_staff_position(1, 4)
                NamedPitch("dss'")

        Returns new named pitch.
        '''
        from abjad.tools import pitchtools
        pitch_number = self.pitch_number + interval
        diatonic_pitch_class_number = self.diatonic_pitch_class_number
        diatonic_pitch_class_number += staff_positions
        diatonic_pitch_class_number %= 7
        class_ = pitchtools.PitchClass
        dictionary = \
            class_._diatonic_pitch_class_number_to_diatonic_pitch_class_name
        diatonic_pitch_class_name = dictionary[diatonic_pitch_class_number]
        return type(self)(pitch_number, diatonic_pitch_class_name)



# -*- coding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.topleveltools import attach


class StringQuartetScoreTemplate(AbjadObject):
    '''String quartet score template.

    ::

        >>> template = templatetools.StringQuartetScoreTemplate()
        >>> score = template()

    ::

        >>> score
        <Score-"String Quartet Score"<<1>>>

    ..  doctest::

        >>> print(format(score))
        \context Score = "String Quartet Score" <<
            \context StaffGroup = "String Quartet Staff Group" <<
                \tag #'first-violin
                \context Staff = "First Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "First Violin Voice" {
                    }
                }
                \tag #'second-violin
                \context Staff = "Second Violin Staff" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin }
                    \set Staff.shortInstrumentName = \markup { Vn. }
                    \context Voice = "Second Violin Voice" {
                    }
                }
                \tag #'viola
                \context Staff = "Viola Staff" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola }
                    \set Staff.shortInstrumentName = \markup { Va. }
                    \context Voice = "Viola Voice" {
                    }
                }
                \tag #'cello
                \context Staff = "Cello Staff" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello }
                    \set Staff.shortInstrumentName = \markup { Vc. }
                    \context Voice = "Cello Voice" {
                    }
                }
            >>
        >>

    Returns score template.
    '''

    ### CLASS VARIABLES ###

    context_name_abbreviations = collections.OrderedDict({
        'vn1': 'First Violin Voice',
        'vn2': 'Second Violin Voice',
        'va': 'Viola Voice',
        'vc': 'Cello Voice',
        })

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls string quartet score template.

        Returns score.
        '''

        # make first violin voice and staff
        first_violin_voice = scoretools.Voice(name='First Violin Voice')
        first_violin_staff = scoretools.Staff(
            [first_violin_voice], name='First Violin Staff')
        clef = indicatortools.Clef('treble')
        attach(clef, first_violin_staff)
        violin = instrumenttools.Violin()
        attach(violin, first_violin_staff)
        tag = indicatortools.LilyPondCommand("tag #'first-violin", 'before')
        attach(tag, first_violin_staff)

        # make second violin voice and staff
        second_violin_voice = scoretools.Voice(name='Second Violin Voice')
        second_violin_staff = scoretools.Staff(
            [second_violin_voice], name='Second Violin Staff')
        clef = indicatortools.Clef('treble')
        attach(clef, second_violin_staff)
        violin = instrumenttools.Violin()
        attach(violin, second_violin_staff)
        tag = indicatortools.LilyPondCommand("tag #'second-violin", 'before')
        attach(tag, second_violin_staff)

        # make viola voice and staff
        viola_voice = scoretools.Voice(name='Viola Voice')
        viola_staff = scoretools.Staff(
            [viola_voice], name='Viola Staff')
        clef = indicatortools.Clef('alto')
        attach(clef, viola_staff)
        viola = instrumenttools.Viola()
        attach(viola, viola_staff)
        tag = indicatortools.LilyPondCommand("tag #'viola", 'before')
        attach(tag, viola_staff)

        # make cello voice and staff
        cello_voice = scoretools.Voice(name='Cello Voice')
        cello_staff = scoretools.Staff(
            [cello_voice], name='Cello Staff')
        clef = indicatortools.Clef('bass')
        attach(clef, cello_staff)
        cello = instrumenttools.Cello()
        attach(cello, cello_staff)
        tag = indicatortools.LilyPondCommand("tag #'cello", 'before')
        attach(tag, cello_staff)

        # make string quartet staff group
        string_quartet_staff_group = scoretools.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            ],
            name='String Quartet Staff Group',
            )

        # make string quartet score
        string_quartet_score = scoretools.Score(
            [string_quartet_staff_group],
            name='String Quartet Score',
            )

        # return string quartet score
        return string_quartet_score



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import systemtools
from abjad.tools.quantizationtools.QSchema import QSchema


class MeasurewiseQSchema(QSchema):
    r'''Concrete QSchema subclass which treats "measures" as its time-step
    unit.

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema()

    Without arguments, it uses smart defaults:

    ::

        >>> print(format(q_schema, 'storage'))
        quantizationtools.MeasurewiseQSchema(
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={
                    2: {
                        2: {
                            2: {
                                2: None,
                                },
                            3: None,
                            },
                        3: None,
                        5: None,
                        7: None,
                        },
                    3: {
                        2: {
                            2: None,
                            },
                        3: None,
                        5: None,
                        },
                    5: {
                        2: None,
                        3: None,
                        },
                    7: {
                        2: None,
                        },
                    11: None,
                    13: None,
                    },
                ),
            tempo=indicatortools.Tempo(
                reference_duration=durationtools.Duration(1, 4),
                units_per_minute=60,
                ),
            time_signature=indicatortools.TimeSignature((4, 4)),
            use_full_measure=False,
            )

    Each time-step in a ``MeasurewiseQSchema`` is composed of four settings:

        * ``search_tree``
        * ``tempo``
        * ``time_signature``
        * ``use_full_measure``

    These settings can be applied as global defaults for the schema via keyword
    arguments, which persist until overridden:

    ::

        >>> search_tree = quantizationtools.UnweightedSearchTree({7: None})
        >>> time_signature = TimeSignature((3, 4))
        >>> tempo = Tempo((1, 4), 54)
        >>> use_full_measure = True
        >>> q_schema = quantizationtools.MeasurewiseQSchema(
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     time_signature=time_signature,
        ...     use_full_measure=use_full_measure,
        ...     )

    All of these settings are self-descriptive, except for
    ``use_full_measure``, which controls whether the measure is subdivided by
    the ``Quantizer`` into beats according to its time signature.

    If ``use_full_measure`` is ``False``, the time-step's measure will be
    divided into units according to its time-signature.  For example, a 4/4
    measure will be divided into 4 units, each having a beatspan of 1/4.

    On the other hand, if ``use_full_measure`` is set to ``True``, the
    time-step's measure will not be subdivided into independent quantization
    units. This usually results in full-measure tuplets.

    The computed value at any non-negative time-step can be found by
    subscripting:

    ::

        >>> index = 0
        >>> for key, value in sorted(q_schema[index].items()):
        ...     print('{}:'.format(key), value)
        ...
        search_tree: UnweightedSearchTree(definition={7: None})
        tempo: 4=54
        time_signature: 3/4
        use_full_measure: True

    ::

        >>> index = 1000
        >>> for key, value in sorted(q_schema[index].items()):
        ...     print('{}:'.format(key), value)
        ...
        search_tree: UnweightedSearchTree(definition={7: None})
        tempo: 4=54
        time_signature: 3/4
        use_full_measure: True

    Per-time-step settings can be applied in a variety of ways.

    Instantiating the schema via ``*arguments`` with a series of either
    ``MeasurewiseQSchemaItem`` instances, or dictionaries which could be used
    to instantiate ``MeasurewiseQSchemaItem`` instances, will apply those
    settings sequentially, starting from time-step ``0``:

    ::

        >>> a = {'search_tree': quantizationtools.UnweightedSearchTree({2: None})}
        >>> b = {'search_tree': quantizationtools.UnweightedSearchTree({3: None})}
        >>> c = {'search_tree': quantizationtools.UnweightedSearchTree({5: None})}

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(a, b, c)

    ::

        >>> q_schema[0]['search_tree']
        UnweightedSearchTree(definition={2: None})

    ::

        >>> q_schema[1]['search_tree']
        UnweightedSearchTree(definition={3: None})

    ::

        >>> q_schema[2]['search_tree']
        UnweightedSearchTree(definition={5: None})

    ::

        >>> q_schema[1000]['search_tree']
        UnweightedSearchTree(definition={5: None})

    Similarly, instantiating the schema from a single dictionary, consisting of
    integer:specification pairs, or a sequence via ``*arguments`` of (integer,
    specification) pairs, allows for applying settings to non-sequential
    time-steps:

    ::

        >>> a = {'time_signature': TimeSignature((7, 32))}
        >>> b = {'time_signature': TimeSignature((3, 4))}
        >>> c = {'time_signature': TimeSignature((5, 8))}

    ::

        >>> settings = {
        ...     2: a,
        ...     4: b,
        ...     6: c,
        ... }

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(settings)

    ::

        >>> q_schema[0]['time_signature']
        TimeSignature((4, 4))

    ::

        >>> q_schema[1]['time_signature']
        TimeSignature((4, 4))

    ::

        >>> q_schema[2]['time_signature']
        TimeSignature((7, 32))

    ::

        >>> q_schema[3]['time_signature']
        TimeSignature((7, 32))

    ::

        >>> q_schema[4]['time_signature']
        TimeSignature((3, 4))

    ::

        >>> q_schema[5]['time_signature']
        TimeSignature((3, 4))

    ::

        >>> q_schema[6]['time_signature']
        TimeSignature((5, 8))

    ::

        >>> q_schema[1000]['time_signature']
        TimeSignature((5, 8))

    The following is equivalent to the above schema definition:

    ::

        >>> q_schema = quantizationtools.MeasurewiseQSchema(
        ...     (2, {'time_signature': TimeSignature((7, 32))}),
        ...     (4, {'time_signature': TimeSignature((3, 4))}),
        ...     (6, {'time_signature': TimeSignature((5, 8))}),
        ...     )

    Return ``MeasurewiseQSchema`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_items',
        '_lookups',
        '_search_tree',
        '_tempo',
        '_time_signature',
        '_use_full_measure',
        )

    _keyword_argument_names = (
        'search_tree',
        'tempo',
        'time_signature',
        'use_full_measure',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments, **keywords):
        from abjad.tools import quantizationtools
        search_tree = keywords.get(
            'search_tree', quantizationtools.UnweightedSearchTree())
        assert isinstance(search_tree, quantizationtools.SearchTree)
        self._search_tree = search_tree
        tempo = keywords.get('tempo', ((1, 4), 60))
        if isinstance(tempo, tuple):
            tempo = indicatortools.Tempo(*tempo)
        self._tempo = tempo
        self._time_signature = indicatortools.TimeSignature(
            keywords.get('time_signature', (4, 4)))
        self._use_full_measure = bool(keywords.get('use_full_measure'))
        QSchema.__init__(self, *arguments, **keywords)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            storage_format_args_values=self.items or (),
            storage_format_kwargs_names=[
                'search_tree',
                'tempo',
                'time_signature',
                'use_full_measure',
                ],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        r'''Item class of measurewise q-schema.

        Returns ``MeasurewiseQSchemaItem``.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.MeasurewiseQSchemaItem

    @property
    def target_class(self):
        r'''Target class of measurewise q-schema.

        Returns ``MeasurewiseQTarget``.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.MeasurewiseQTarget

    @property
    def target_item_class(self):
        r'''Target item class of measurewise q-schema.

        Returns ``QTargetMeasure``.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetMeasure

    @property
    def time_signature(self):
        r'''Default time signature of measurewise q-schema.

        Returns time signature.
        '''
        return self._time_signature

    @property
    def use_full_measure(self):
        r'''The full-measure-as-beatspan default.

        Returns true or false.
        '''
        return self._use_full_measure



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_ChordSuspension__init_empty_01():

    chord_suspension = tonalanalysistools.ChordSuspension()

    assert chord_suspension.start is None
    assert chord_suspension.stop is None



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_Timespan___xor___01():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, -5)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, -5),
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___xor___02():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 0)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 0),
        timespantools.Timespan(0, 15)
    ])

def test_timespantools_Timespan___xor___03():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 5)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 0),
        timespantools.Timespan(5, 15)
    ])

def test_timespantools_Timespan___xor___04():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 15)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 0)
    ])

def test_timespantools_Timespan___xor___05():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 25)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(-10, 0),
        timespantools.Timespan(15, 25)
    ])

def test_timespantools_Timespan___xor___06():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 10)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(10, 15)
    ])

def test_timespantools_Timespan___xor___07():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 15)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
    ])

def test_timespantools_Timespan___xor___08():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 10)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 5),
        timespantools.Timespan(10, 15)
    ])

def test_timespantools_Timespan___xor___09():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 15)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 5)
    ])

def test_timespantools_Timespan___xor___10():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 25)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(15, 25)
    ])

def test_timespantools_Timespan___xor___11():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 25)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 5),
        timespantools.Timespan(15, 25)
    ])

def test_timespantools_Timespan___xor___12():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(15, 25)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15),
        timespantools.Timespan(15, 25)
    ])

def test_timespantools_Timespan___xor___13():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(20, 25)
    result = timespan_1 ^ timespan_2
    assert result == timespantools.TimespanList([
        timespantools.Timespan(0, 15),
        timespantools.Timespan(20, 25)
    ])



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.datastructuretools import TypedOrderedDict


class PartitionTable(TypedOrderedDict):
    r'''Partition table.

    ..  container:: example

        Partitions integers roughly ``1:1`` (restricted to assignable values)
        with larger half on left:

        ::

            >>> table = rhythmmakertools.PartitionTable([
            ...     (2, [1, 1]),
            ...     (3, [2, 1]),
            ...     (5, [3, 2]),
            ...     (7, [4, 3]),
            ...     ])

        ::

            >>> f(table)
            abjad.rhythmmakertools.PartitionTable(
                [
                    (
                        2,
                        mathtools.NonreducedRatio((1, 1)),
                        ),
                    (
                        3,
                        mathtools.NonreducedRatio((2, 1)),
                        ),
                    (
                        5,
                        mathtools.NonreducedRatio((3, 2)),
                        ),
                    (
                        7,
                        mathtools.NonreducedRatio((4, 3)),
                        ),
                    ]
                )

    ..  container:: example

        Partitions integers roughly ``1:1`` (restricted to assignable values)
        with larger half on right:

        ::

            >>> table = rhythmmakertools.PartitionTable([
            ...     (2, [1, 1]),
            ...     (3, [1, 2]),
            ...     (5, [2, 3]),
            ...     (7, [3, 4]),
            ...     ])

        ::

            >>> f(table)
            abjad.rhythmmakertools.PartitionTable(
                [
                    (
                        2,
                        mathtools.NonreducedRatio((1, 1)),
                        ),
                    (
                        3,
                        mathtools.NonreducedRatio((1, 2)),
                        ),
                    (
                        5,
                        mathtools.NonreducedRatio((2, 3)),
                        ),
                    (
                        7,
                        mathtools.NonreducedRatio((3, 4)),
                        ),
                    ]
                )

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        )

    _publish_storage_format = True

    ### PRIVATE METHODS ###

    def _item_coercer(self, item):
        item = mathtools.NonreducedRatio(item)
        return item

    ### PUBLIC METHODS ###

    def respell_division(self, division):
        r'''Respells `division` according to partition table.

        ..  container:: example

            Respells divisions according to table:

            ::

                >>> table = rhythmmakertools.PartitionTable([
                ...     (2, [1, 1]),
                ...     (3, [2, 1]),
                ...     (5, [3, 2]),
                ...     (7, [4, 3]),
                ...     ])

            ::

                >>> for numerator in range(1, 15):
                ...     division = mathtools.NonreducedFraction(numerator, 4)
                ...     result = table.respell_division(division)
                ...     print((division, result))
                ...
                (NonreducedFraction(1, 4), [NonreducedFraction(1, 4)])
                (NonreducedFraction(2, 4), [NonreducedFraction(1, 4), NonreducedFraction(1, 4)])
                (NonreducedFraction(3, 4), [NonreducedFraction(2, 4), NonreducedFraction(1, 4)])
                (NonreducedFraction(4, 4), [NonreducedFraction(2, 4), NonreducedFraction(2, 4)])
                (NonreducedFraction(5, 4), [NonreducedFraction(3, 4), NonreducedFraction(2, 4)])
                (NonreducedFraction(6, 4), [NonreducedFraction(4, 4), NonreducedFraction(2, 4)])
                (NonreducedFraction(7, 4), [NonreducedFraction(4, 4), NonreducedFraction(3, 4)])
                (NonreducedFraction(8, 4), [NonreducedFraction(4, 4), NonreducedFraction(4, 4)])
                (NonreducedFraction(9, 4), [NonreducedFraction(6, 4), NonreducedFraction(3, 4)])
                (NonreducedFraction(10, 4), [NonreducedFraction(6, 4), NonreducedFraction(4, 4)])
                (NonreducedFraction(11, 4), [NonreducedFraction(11, 4)])
                (NonreducedFraction(12, 4), [NonreducedFraction(8, 4), NonreducedFraction(4, 4)])
                (NonreducedFraction(13, 4), [NonreducedFraction(13, 4)])
                (NonreducedFraction(14, 4), [NonreducedFraction(8, 4), NonreducedFraction(6, 4)])

        ..  container:: example

            Respells divisions according to another table:

            ::

                >>> table = rhythmmakertools.PartitionTable([
                ...     (5, [1, 3, 1]),
                ...     (7, [1, 2, 4]),
                ...     ])

            ::

                >>> for numerator in range(1, 15):
                ...     division = mathtools.NonreducedFraction(numerator, 4)
                ...     result = table.respell_division(division)
                ...     print((division, result))
                ...
                (NonreducedFraction(1, 4), [NonreducedFraction(1, 4)])
                (NonreducedFraction(2, 4), [NonreducedFraction(2, 4)])
                (NonreducedFraction(3, 4), [NonreducedFraction(3, 4)])
                (NonreducedFraction(4, 4), [NonreducedFraction(4, 4)])
                (NonreducedFraction(5, 4), [NonreducedFraction(1, 4), NonreducedFraction(3, 4), NonreducedFraction(1, 4)])
                (NonreducedFraction(6, 4), [NonreducedFraction(6, 4)])
                (NonreducedFraction(7, 4), [NonreducedFraction(1, 4), NonreducedFraction(2, 4), NonreducedFraction(4, 4)])
                (NonreducedFraction(8, 4), [NonreducedFraction(8, 4)])
                (NonreducedFraction(9, 4), [NonreducedFraction(9, 4)])
                (NonreducedFraction(10, 4), [NonreducedFraction(2, 4), NonreducedFraction(6, 4), NonreducedFraction(2, 4)])
                (NonreducedFraction(11, 4), [NonreducedFraction(11, 4)])
                (NonreducedFraction(12, 4), [NonreducedFraction(12, 4)])
                (NonreducedFraction(13, 4), [NonreducedFraction(13, 4)])
                (NonreducedFraction(14, 4), [NonreducedFraction(2, 4), NonreducedFraction(4, 4), NonreducedFraction(8, 4)])

        Returns list of new divisions.
        '''
        division = mathtools.NonreducedFraction(division)
        result = [division]
        divisors = mathtools.divisors(division.numerator)
        for numerator, ratio in reversed(list(self.items())):
            if numerator in divisors:
                result = [division * _ for _ in ratio.multipliers]
                break
        result = [mathtools.NonreducedFraction(_) for _ in result]
        return result



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class BendAfter(AbjadValueObject):
    r'''Fall or doit.

    ..  container:: example

        **Example 1.** A fall:

        ::

            >>> note = Note("c'4")
            >>> bend = indicatortools.BendAfter(-4)
            >>> attach(bend, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4 - \bendAfter #'-4.0

    ..  container:: example

        **Example 2.** A doit:

        ::

            >>> note = Note("c'4")
            >>> bend = indicatortools.BendAfter(2)
            >>> attach(bend, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            c'4 - \bendAfter #'2.0

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_bend_amount',
        '_default_scope',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self, bend_amount=-4):
        if isinstance(bend_amount, type(self)):
            bend_amount = bend_amount.bend_amount
        bend_amount = float(bend_amount)
        self._bend_amount = bend_amount
        self._default_scope = None

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of bend after.

        ..  container:: example

            ::

                >>> str(indicatortools.BendAfter())
                "- \\bendAfter #'-4.0"

        Returns string.
        '''
        return r"- \bendAfter #'{}".format(self.bend_amount)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return str(self.bend_amount)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def bend_amount(self):
        r'''Gets bend amount of bend after.

        ..  container:: example

            **Example 1.** Fall:

            ::

                >>> bend = indicatortools.BendAfter(-4)
                >>> bend.bend_amount
                -4.0

        ..  container:: example

            **Example 2.** Doit:

            ::

                >>> bend = indicatortools.BendAfter(2)
                >>> bend.bend_amount
                2.0 

        Returns float.
        '''
        return self._bend_amount

    @property
    def default_scope(self):
        r'''Gets default scope of bend after.

        ..  container:: example

            >>> bend = indicatortools.BendAfter(-4)
            >>> bend.default_scope is None
            True

        Returns none.
        '''
        return self._default_scope



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class AltoSaxophone(Instrument):
    r'''An alto saxophone.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> alto_saxophone = instrumenttools.AltoSaxophone()
        >>> attach(alto_saxophone, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { "Alto saxophone" }
            \set Staff.shortInstrumentName = \markup { "Alt. sax." }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='alto saxophone',
        short_instrument_name='alt. sax.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=None,
        pitch_range='[Db3, A5]',
        sounding_pitch_of_written_middle_c='Eb3',
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'wind player',
            'reed player',
            'single reed player',
            'saxophonist',
            ])
        self._is_primary_instrument = True

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats alto sax.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            ::

                >>> alto_sax = instrumenttools.AltoSaxophone()
                >>> print(format(alto_sax))
                instrumenttools.AltoSaxophone(
                    instrument_name='alto saxophone',
                    short_instrument_name='alt. sax.',
                    instrument_name_markup=markuptools.Markup(
                        contents=['Alto saxophone'],
                        ),
                    short_instrument_name_markup=markuptools.Markup(
                        contents=['Alt. sax.'],
                        ),
                    allowable_clefs=instrumenttools.ClefList(
                        [
                            indicatortools.Clef(
                                name='treble',
                                ),
                            ]
                        ),
                    pitch_range=pitchtools.PitchRange(
                        range_string='[Db3, A5]',
                        ),
                    sounding_pitch_of_written_middle_c=pitchtools.NamedPitch('ef'),
                    )

        Returns string.
        '''
        superclass = super(AltoSaxophone, self)
        return superclass.__format__(format_specification=format_specification)

    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets alto saxophone's allowable clefs.

        ..  container:: example

            ::

                >>> alto_saxophone.allowable_clefs
                ClefList([Clef(name='treble')])

            ::

                >>> show(alto_saxophone.allowable_clefs) # doctest: +SKIP

        Returns clef list.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets alto saxophone's name.

        ..  container:: example

            ::

                >>> alto_saxophone.instrument_name
                'alto saxophone'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets alto saxophone's instrument name markup.

        ..  container:: example

            ::

                >>> alto_saxophone.instrument_name_markup
                Markup(contents=['Alto saxophone'])

            ::

                >>> show(alto_saxophone.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets alto saxophone's range.

        ..  container:: example

            ::

                >>> alto_saxophone.pitch_range
                PitchRange(range_string='[Db3, A5]')

            ::

                >>> show(alto_saxophone.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets alto saxophone's short instrument name.

        ..  container:: example

            ::

                >>> alto_saxophone.short_instrument_name
                'alt. sax.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets alto saxophone's short instrument name markup.

        ..  container:: example

            ::

                >>> alto_saxophone.short_instrument_name_markup
                Markup(contents=['Alt. sax.'])

            ::

                >>> show(alto_saxophone.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of alto saxophone's written middle C.

        ..  container:: example

            ::

                >>> alto_saxophone.sounding_pitch_of_written_middle_c
                NamedPitch('ef')

            ::

                >>> show(alto_saxophone.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Leaf__multiplied_duration_01():
    r'''Mulplied duration == written * multiplier.
    '''

    note = Note("c'4")
    attach(Multiplier(1, 2), note)
    assert note._multiplied_duration == Duration(1, 8)


def test_scoretools_Leaf__multiplied_duration_02():
    r'''Mulplied duration equals duration when multiplier is none.
    '''

    note = Note("c'4")
    assert note._multiplied_duration == Duration(1, 4)


def test_scoretools_Leaf__multiplied_duration_03():
    r'''Attach multiplier and then detach multiplier.
    '''

    note = Note("c'4")
    note.written_duration = Duration(3, 8)
    attach(Multiplier(2, 3), note)

    assert note.written_duration == Duration(3, 8)
    assert inspect_(note).get_indicator(Multiplier) == Multiplier(2, 3)
    assert note._multiplied_duration == Duration(1, 4)

    note.written_duration = Duration(1, 4)
    detach(Multiplier, note)

    assert note.written_duration == Duration(1, 4)
    assert inspect_(note).get_indicators(Multiplier) == ()
    assert note._multiplied_duration == Duration(1, 4)



# -*- coding: utf-8 -*-
from abjad.tools.quantizationtools.AttackPointOptimizer \
    import AttackPointOptimizer


class NullAttackPointOptimizer(AttackPointOptimizer):
    r'''Concrete ``AttackPointOptimizer`` subclass which performs
    no attack point optimization.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        r'''Calls null attack-point optimizer.
        '''
        pass



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitch___slots___01():
    r'''Numbered pitches are immutable.
    '''

    numbered_pitch = pitchtools.NumberedPitch(13)
    assert pytest.raises(AttributeError, "numbered_pitch.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo_rewrite_duration_01():

    tempo_1 = Tempo(Duration(1, 4), 60)
    new_tempo = Tempo(Duration(1, 4), 90)

    result = tempo_1.rewrite_duration(Duration(1, 8), new_tempo)
    assert result == Duration(3, 16)

    result = tempo_1.rewrite_duration(Duration(1, 12), new_tempo)
    assert result == Duration(1, 8)

    result = tempo_1.rewrite_duration(Duration(1, 16), new_tempo)
    assert result == Duration(3, 32)

    result = tempo_1.rewrite_duration(Duration(1, 24), new_tempo)
    assert result == Duration(1, 16)



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___len___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(preprolated_duration=1)

    container = rhythmtreetools.RhythmTreeContainer(
        preprolated_duration=1, children=[leaf_a, subcontainer, leaf_d])

    assert len(container) == 3



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_Scale_make_score_01():

    scale = tonalanalysistools.Scale('E', 'major')
    score = scale.make_score()

    assert format(score) == stringtools.normalize(
        r'''
        \new Score \with {
            tempoWholesPerMinute = #(ly:make-moment 30 1)
        } <<
            \new Staff {
                \key e \major
                e'8
                fs'8
                gs'8
                a'8
                b'8
                cs''8
                ds''8
                e''8
                ds''8
                cs''8
                b'8
                a'8
                gs'8
                fs'8
                e'4
            }
        >>
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet___hash___01():
    r'''Named pitch-class sets are hashable.
    '''

    named_pitch_class_set = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),])

    assert hash(named_pitch_class_set) == hash(repr(named_pitch_class_set))


def test_pitchtools_PitchClassSet___hash___02():
    r'''Pitch class sets are hashable.
    '''

    pitch_class_set = pitchtools.PitchClassSet([0, 1, 2])

    assert hash(pitch_class_set) == hash(repr(pitch_class_set))



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__contexts__context_ids_01():

    target = Staff(scoretools.make_notes([0, 2, 4, 5, 7], (1, 8)))
    target.name = 'foo'

    assert format(target) == stringtools.normalize(
        r'''
        \context Staff = "foo" {
            c'8
            d'8
            e'8
            f'8
            g'8
        }
        '''
        )

    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import PitchRange


def test_pitchtools_PitchRangeList___repr___01():

    pitch_ranges_1 = pitchtools.PitchRangeList(['[A0, C8]', '[C3, F#5]'])
    pitch_ranges_2 = pitchtools.PitchRangeList(pitch_ranges_1)

    assert pitch_ranges_1 == pitch_ranges_2



# -*- coding: utf-8 -*-
from abjad.tools import lilypondparsertools


def test_lilypondparsertools_LilyPondParser_accidentals_cautionary_01():

    string = "{ c?4 }"
    parsed = lilypondparsertools.LilyPondParser()(string)

    assert parsed[0].note_head.is_cautionary == True
    assert format(parsed[0]) == 'c?4'

def test_lilypondparsertools_LilyPondParser_accidentals_cautionary_02():

    string = "{ <c? e g??>4 }"
    parsed = lilypondparsertools.LilyPondParser()(string)

    assert parsed[0].note_heads[0].is_cautionary == True
    assert parsed[0].note_heads[1].is_cautionary == False
    assert parsed[0].note_heads[2].is_cautionary == True
    assert format(parsed[0]) == '<c? e g?>4'



# -*- coding: utf-8 -*-
from abjad.tools import patterntools


def silence_last(n=1, inverted=None, use_multimeasure_rests=None):
    r'''Makes silence mask that matches the last `n` indices.

    ..  container:: example

        Silences last division:

        ::

            >>> mask = rhythmmakertools.silence_last()

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[-1],
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    r4.
                }
            }

    ..  container:: example

        Silences last two divisions:

        ::

            >>> mask = rhythmmakertools.silence_last(n=2)

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[-2, -1],
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    r4..
                }
                {
                    \time 3/8
                    r4.
                }
            }

    ..  container:: example

        Silences no last divisions:

        ::

            >>> mask = rhythmmakertools.silence_last(n=0)

        ::

            >>> f(mask)
            rhythmmakertools.SilenceMask(
                pattern=patterntools.Pattern(
                    indices=[],
                    ),
                )

        ::

            >>> rhythm_maker = rhythmmakertools.NoteRhythmMaker(
            ...     division_masks=[mask],
            ...     )
            >>> divisions = [(7, 16), (3, 8), (7, 16), (3, 8)]
            >>> selections = rhythm_maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     selections,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> f(lilypond_file[Staff])
            \new RhythmicStaff {
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
                {
                    \time 7/16
                    c'4..
                }
                {
                    \time 3/8
                    c'4.
                }
            }

    Returns silence mask.
    '''
    from abjad.tools import rhythmmakertools
    indices = list(reversed(range(-1, -n-1, -1)))
    pattern = patterntools.Pattern(
        indices=indices,
        inverted=inverted,
        )
    mask = rhythmmakertools.SilenceMask(
        pattern=pattern,
        use_multimeasure_rests=use_multimeasure_rests,
        )
    return mask



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTHorizontalRule(TreeNode):
    r'''A ReST horizontal rule.

    ::

        >>> rule = documentationtools.ReSTHorizontalRule()
        >>> rule
        ReSTHorizontalRule()

    ::

        >>> print(rule.rest_format)
        --------

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        return ['--------']

    ### PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        r'''ReST format of ReSt horizontal rule.

        Returns text.
        '''
        return '\n'.join(self._rest_format_contributions)



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_neighbor_notes_01():

    notes = scoretools.make_notes([0, 2, 4, 2, 0], [(1, 4)])
    staff = Staff(notes)

    selection = tonalanalysistools.select(staff[:])
    result = selection.analyze_neighbor_notes()
    result == [False, False, True, False, False]



# -*- coding: utf-8 -*-
from abjad import *
import copy


def test_scoretools_NoteHead___copy___01():

    note_head_1 = scoretools.NoteHead("cs''")
    note_head_1.is_cautionary = True
    note_head_1.is_forced = True
    note_head_1.tweak.color = 'red'
    note_head_1.tweak.font_size = -2

    note_head_2 = copy.copy(note_head_1)

    assert isinstance(note_head_1, scoretools.NoteHead)
    assert isinstance(note_head_2, scoretools.NoteHead)
    assert note_head_1 == note_head_2
    assert note_head_1 is not note_head_2
    assert note_head_1.is_cautionary == note_head_2.is_cautionary
    assert note_head_1.is_forced == note_head_2.is_forced
    assert note_head_1.tweak == note_head_2.tweak
    assert note_head_1.tweak is not note_head_2.tweak



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin__is_hairpin_shape_string_01():

    assert Hairpin._is_hairpin_shape_string('<')
    assert Hairpin._is_hairpin_shape_string('>')


def test_spannertools_Hairpin__is_hairpin_shape_string_02():

    assert not Hairpin._is_hairpin_shape_string('@')



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitchClass___init___01():
    r'''Initialize from name.
    '''

    assert pitchtools.NamedPitchClass('c') == 'c'
    assert pitchtools.NamedPitchClass('cs') == 'cs'
    assert pitchtools.NamedPitchClass('cf') == 'cf'
    assert pitchtools.NamedPitchClass('cqs') == 'cqs'
    assert pitchtools.NamedPitchClass('cqf') == 'cqf'


def test_pitchtools_NamedPitchClass___init___02():
    r'''Initialize from other named pitch-class instance.
    '''

    npc = pitchtools.NamedPitchClass('c')
    new = pitchtools.NamedPitchClass(npc)

    assert new == npc
    assert new is not npc


def test_pitchtools_NamedPitchClass___init___03():
    r'''Initialize from note-head instance.
    '''

    chord = Chord([0, 2, 3], (1, 4))
    note_head = chord.note_heads[0]
    npc = pitchtools.NamedPitchClass(note_head)

    assert npc == pitchtools.NamedPitchClass('c')



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    expected_files = [
        'test_score/test_score/segments/.gitignore',
        'test_score/test_score/segments/__init__.py',
        'test_score/test_score/segments/metadata.json',
        'test_score/test_score/segments/test_segment/__init__.py',
        'test_score/test_score/segments/test_segment/definition.py',
        'test_score/test_score/segments/test_segment/illustration.ly',
        'test_score/test_score/segments/test_segment/illustration.pdf',
        'test_score/test_score/segments/test_segment/metadata.json',
        ]

    if platform.system().lower() == 'windows':
        expected_files = [_.replace('/', os.path.sep) for _ in expected_files]

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_one_segment(self, open_file_mock):
        self.create_score()
        segment_path = self.create_segment('test_segment')
        self.illustrate_segment('test_segment')
        pdf_path = segment_path.joinpath('illustration.pdf')
        assert pdf_path.exists()
        pdf_path.unlink()
        script = commandlinetools.ManageSegmentScript()
        command = ['--render', 'test_segment']
        with systemtools.RedirectedStreams(stdout=self.string_io):
            with systemtools.TemporaryDirectoryChange(str(self.score_path)):
                try:
                    script(command)
                except SystemExit as e:
                    raise RuntimeError('SystemExit: {}'.format(e.code))
        self.compare_captured_output(r'''
        Rendering candidates: 'test_segment' ...
            Reading test_score/segments/metadata.json ... OK!
        Rendering test_score/segments/test_segment/
            Writing test_score/segments/test_segment/illustration.pdf ... OK!
                LilyPond runtime: ... second...
            Rendered test_score/segments/test_segment/
        '''.replace('/', os.path.sep))
        self.compare_path_contents(self.segments_path, self.expected_files)



# -*- coding: utf-8 -*-


def factors(n):
    r'''Gets prime factors less than or equal to `n` .

    ..  container:: example

        ::

            >>> mathtools.factors(84)
            [2, 2, 3, 7]

        ::

            >>> for n in range(10, 20):
            ...   print(n, mathtools.factors(n))
            ... 
            10 [2, 5]
            11 [11]
            12 [2, 2, 3]
            13 [13]
            14 [2, 7]
            15 [3, 5]
            16 [2, 2, 2, 2]
            17 [17]
            18 [2, 3, 3]
            19 [19]

    `n` must be a positive integer.

    Returns factors in increasing order.

    Returns list of positive integers.
    '''
    from abjad.tools import mathtools
    if not mathtools.is_positive_integer(n):
        message = 'must be positive integer: {!r}.'
        message = message.format(n)
        raise TypeError(message)
    factor = 2
    factors = []
    while 1 < n:
        if n % factor == 0:
            factors.append(factor)
            n = n / factor
        else:
            factor = factor + 1
    return factors



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_repeated_notes_01():
    r'''Allow nonassignable durations.
    '''

    voice = Voice(scoretools.make_repeated_notes(2, (5, 16)))

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'4 ~
            c'16
            c'4 ~
            c'16
        }
        '''
        )



# -*- coding: utf-8 -*-
import collections
import functools
import math
import numbers
from abjad import Fraction
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import schemetools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject
from abjad.tools.topleveltools import new


@functools.total_ordering
class Tempo(AbjadValueObject):
    r'''Tempo.

    ..  container:: example

        **Example 1.** Integer-valued tempo:

        ::

            >>> score = Score([])
            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> score.append(staff)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \tempo 4=90
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>

    ..  container:: example

        **Example 2.** Float-valued tempo:

        ::

            >>> score = Score([])
            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> score.append(staff)
            >>> tempo = Tempo(Duration(1, 4), 90.1)
            >>> attach(tempo, staff[0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \tempo \markup {
                        \scale
                            #'(0.75 . 0.75)
                            \score
                                {
                                    \new Score \with {
                                        \override SpacingSpanner.spacing-increment = #0.5
                                        proportionalNotationDuration = ##f
                                    } <<
                                        \new RhythmicStaff \with {
                                            \remove Time_signature_engraver
                                            \remove Staff_symbol_engraver
                                            \override Stem.direction = #up
                                            \override Stem.length = #5
                                            \override TupletBracket.bracket-visibility = ##t
                                            \override TupletBracket.direction = #up
                                            \override TupletBracket.padding = #1.25
                                            \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                            \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                            tupletFullLength = ##t
                                        } {
                                            c'4
                                        }
                                    >>
                                    \layout {
                                        indent = #0
                                        ragged-right = ##t
                                    }
                                }
                        =
                        \general-align
                            #Y
                            #-0.5
                            90.1
                        }
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>

    ..  container:: example

        **Example 3.** Rational-valued tempo:

        ::

            >>> score = Score([])
            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> score.append(staff)
            >>> tempo = Tempo(Duration(1, 4), Fraction(181, 2))
            >>> attach(tempo, staff[0])
            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \tempo \markup {
                        \scale
                            #'(0.75 . 0.75)
                            \score
                                {
                                    \new Score \with {
                                        \override SpacingSpanner.spacing-increment = #0.5
                                        proportionalNotationDuration = ##f
                                    } <<
                                        \new RhythmicStaff \with {
                                            \remove Time_signature_engraver
                                            \remove Staff_symbol_engraver
                                            \override Stem.direction = #up
                                            \override Stem.length = #5
                                            \override TupletBracket.bracket-visibility = ##t
                                            \override TupletBracket.direction = #up
                                            \override TupletBracket.padding = #1.25
                                            \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                            \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                            tupletFullLength = ##t
                                        } {
                                            c'4
                                        }
                                    >>
                                    \layout {
                                        indent = #0
                                        ragged-right = ##t
                                    }
                                }
                        =
                        \raise
                            #-0.5
                            {
                                90
                                \tiny
                                    \fraction
                                        1
                                        2
                            }
                        }
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_custom_markup',
        '_default_scope',
        '_reference_duration',
        '_textual_indication',
        '_units_per_minute',
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(
        self,
        reference_duration=None,
        units_per_minute=None,
        textual_indication=None,
        custom_markup=None,
        ):
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        self._default_scope = scoretools.Score
        assert isinstance(textual_indication, (str, type(None)))
        arguments = (reference_duration, units_per_minute, textual_indication)
        if all(_ is None for _ in arguments):
            reference_duration = (1, 4)
            units_per_minute = 60
        if reference_duration:
            reference_duration = durationtools.Duration(reference_duration)
        prototype = (
            int,
            float,
            Fraction,
            collections.Sequence,
            type(None),
            )
        assert isinstance(units_per_minute, prototype)
        if isinstance(units_per_minute, collections.Sequence):
            assert len(units_per_minute) == 2
            prototype = (int, float, durationtools.Duration)
            assert all(isinstance(x, prototype) for x in units_per_minute)
            units_per_minute = tuple(sorted(units_per_minute))
        if isinstance(units_per_minute, float):
            units_per_minute = mathtools.integer_equivalent_number_to_integer(
                units_per_minute)
        self._reference_duration = reference_duration
        self._textual_indication = textual_indication
        self._units_per_minute = units_per_minute
        if custom_markup is not None:
            assert isinstance(custom_markup, markuptools.Markup), repr(
                custom_markup)
        self._custom_markup = custom_markup

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds tempo to `argument`.

        ..  container:: example

            **Example 1.** Adds one tempo to another:

            ::

                >>> Tempo(Duration(1, 4), 60) + Tempo(Duration(1, 4), 90)
                Tempo(reference_duration=Duration(1, 4), units_per_minute=150)

        ..  container:: example

            **Example 2.** Returns none when `argument` is not a tempo:

            ::

                >>> Tempo(Duration(1, 4), 60) + 90 is None
                True

        Returns new tempo or none.
        '''
        if isinstance(argument, type(self)):
            if self.is_imprecise or argument.is_imprecise:
                raise ImpreciseTempoError
            new_quarters_per_minute = \
                self.quarters_per_minute + argument.quarters_per_minute
            minimum_denominator = \
                min((self.reference_duration.denominator, argument.reference_duration.denominator))
            nonreduced_fraction = \
                mathtools.NonreducedFraction(new_quarters_per_minute / 4)
            nonreduced_fraction = \
                nonreduced_fraction.with_denominator(minimum_denominator)
            new_units_per_minute, new_reference_duration_denominator = \
                nonreduced_fraction.pair
            new_reference_duration = \
                durationtools.Duration(1, new_reference_duration_denominator)
            new_tempo = type(self)(
                new_reference_duration,
                new_units_per_minute,
                )
            return new_tempo

    def __div__(self, argument):
        r'''Divides tempo by `argument`.

        ..  container:: example

            **Example 1.** Divides tempo by number:

            ::

                >>> Tempo(Duration(1, 4), 60) / 2
                Tempo(reference_duration=Duration(1, 4), units_per_minute=30)

        ..  container:: example

            **Example 2.** Divides tempo by other tempo:

            ::

                >>> Tempo(Duration(1, 4), 60) / Tempo(Duration(1, 4), 40)
                Multiplier(3, 2)

        Returns new tempo or multiplier.
        '''
        if self.is_imprecise:
            raise ImpreciseTempoError
        if getattr(argument, 'is_imprecise', False):
            raise ImpreciseTempoError
        if isinstance(argument, type(self)):
            result = self.quarters_per_minute / argument.quarters_per_minute
            return durationtools.Multiplier(result)
        elif isinstance(argument, numbers.Number):
            units_per_minute = self.units_per_minute / argument
            result = new(self, units_per_minute=units_per_minute)
            return result
        else:
            message = 'must be number or tempo indication: {!r}.'
            message = message.format(argument)
            raise TypeError(message)

    def __format__(self, format_specification=''):
        r'''Formats tempo.

        Set `format_specification` to `''`', `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ..  container:: example

            **Example 1.** Without custom markup:

            ::

                >>> tempo = Tempo((1, 4), 84, 'Allegro')
                >>> print(format(tempo))
                indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4),
                    units_per_minute=84,
                    textual_indication='Allegro',
                    )

        ..  container:: example

            **Example 2.** With custom markup:

            ::

                >>> markup = Markup(r'\italic { Allegro }')
                >>> tempo = Tempo((1, 4), 84, custom_markup=markup)
                >>> print(format(tempo))
                indicatortools.Tempo(
                    reference_duration=durationtools.Duration(1, 4),
                    units_per_minute=84,
                    custom_markup=markuptools.Markup(
                        contents=[
                            markuptools.MarkupCommand(
                                'italic',
                                ['Allegro']
                                ),
                            ],
                        ),
                    )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        elif format_specification == 'lilypond':
            return self._get_lilypond_format()
        return str(self)

    def __lt__(self, argument):
        r'''Is true when `argument` is a tempo with quarters per minute greater than
        that of this tempo. Otherwise false.

        Returns true or false.
        '''
        assert isinstance(argument, type(self)), repr(argument)
        return self.quarters_per_minute < argument.quarters_per_minute

    def __mul__(self, multiplier):
        r'''Multiplies tempo by `multiplier`.

        ..  container:: example

            **Example 1.** Doubles tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 84)
                >>> 2 * tempo
                Tempo(reference_duration=Duration(1, 4), units_per_minute=168)

        ..  container:: example

            **Example 2.** Triples tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 84)
                >>> 3 * tempo
                Tempo(reference_duration=Duration(1, 4), units_per_minute=252)

        Returns new tempo.
        '''
        if not isinstance(multiplier, (int, float, durationtools.Duration)):
            return
        if self.is_imprecise:
            raise ImpreciseTempoError
        new_units_per_minute = multiplier * self.units_per_minute
        new_reference_duration = durationtools.Duration(
            self.reference_duration)
        new_tempo = type(self)(
            reference_duration=new_reference_duration,
            units_per_minute=new_units_per_minute,
            )
        return new_tempo

    def __rmul__(self, multiplier):
        r'''Multiplies `multiplier` by tempo.

        ..  container::: example

            **Example 1.** Doubles tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 84)
                >>> tempo * 2
                Tempo(reference_duration=Duration(1, 4), units_per_minute=168)

        ..  container::: example

            **Example 2.** Triples tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 84)
                >>> tempo * 3
                Tempo(reference_duration=Duration(1, 4), units_per_minute=252)

        Returns new tempo.
        '''
        if not isinstance(multiplier, (int, float, durationtools.Duration)):
            return
        if self.is_imprecise:
            raise ImpreciseTempoError
        new_units_per_minute = multiplier * self.units_per_minute
        new_reference_duration = durationtools.Duration(
            self.reference_duration)
        new_tempo = type(self)(
            reference_duration=new_reference_duration,
            units_per_minute=new_units_per_minute,
            )
        return new_tempo

    def __str__(self):
        r'''Gets string representation of tempo.

        ..  container:: example

            **Example 1.** Integer-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> str(tempo)
                '4=90'

        ..  container:: example

            **Example 2.** Float-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90.1)
                >>> str(tempo)
                '4=90.1'

        ..  container:: example

            **Example 3.** Rational-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), Fraction(181, 2))
                >>> str(tempo)
                '4=90+1/2'

        ..  container:: example

            **Example 4.** Ranged tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), (90, 96))
                >>> str(tempo)
                '4=90-96'

        Returns string.
        '''
        #return self._equation or self.textual_indication
        if self.textual_indication is not None:
            string = self.textual_indication
        elif isinstance(self.units_per_minute, (int, float)):
            string = '{}={}'
            string = string.format(self._dotted, self.units_per_minute)
        elif (isinstance(self.units_per_minute, Fraction) and
            not mathtools.is_integer_equivalent_number(self.units_per_minute)):
            integer_part = int(self.units_per_minute)
            remainder = self.units_per_minute - integer_part
            remainder = Fraction(remainder)
            string = '{}={}+{}'
            string = string.format(self._dotted, integer_part, remainder)
        elif (isinstance(self.units_per_minute, Fraction) and
            mathtools.is_integer_equivalent_number(self.units_per_minute)):
            string = '{}={}'
            integer = int(self.units_per_minute)
            string = string.format(self._dotted, integer)
        elif isinstance(self.units_per_minute, tuple):
            string = '{}={}-{}'
            string = string.format(
                self._dotted,
                self.units_per_minute[0],
                self.units_per_minute[1],
                )
        else:
            message = 'unknown: {!r}.'
            message = message.format(self.units_per_minute)
            raise TypeError(message)
        return string

    def __sub__(self, argument):
        r'''Subtracts `argument` from tempo.

        ..  container:: example

            **Example 1.** Same reference reference durations:
            ::

                >>> tempo_1 = Tempo(Duration(1, 4), 90)
                >>> tempo_2 = Tempo(Duration(1, 4), 60)
                >>> tempo_1 - tempo_2
                Tempo(reference_duration=Duration(1, 4), units_per_minute=30)

        ..  container:: example

            **Example 2.** Different reference durations:
            ::

                >>> tempo_1 = Tempo(Duration(1, 4), 90)
                >>> tempo_2 = Tempo(Duration(1, 2), 90)
                >>> tempo_1 - tempo_2
                Tempo(reference_duration=Duration(1, 4), units_per_minute=45)

        Returns new tempo.
        '''
        if not isinstance(argument, type(self)):
            message = 'must be tempo: {!r}.'
            message = message.format(argument)
            raise Exception(message)
        if self.is_imprecise or argument.is_imprecise:
            raise ImpreciseTempoError
        new_quarters_per_minute = self.quarters_per_minute - \
            argument.quarters_per_minute
        minimum_denominator = min((
            self.reference_duration.denominator,
            argument.reference_duration.denominator,
            ))
        nonreduced_fraction = mathtools.NonreducedFraction(
            new_quarters_per_minute / 4)
        nonreduced_fraction = nonreduced_fraction.with_denominator(
            minimum_denominator)
        new_units_per_minute, new_reference_duration_denominator = \
            nonreduced_fraction.pair
        new_reference_duration = durationtools.Duration(
            1, new_reference_duration_denominator)
        new_tempo = type(self)(
            reference_duration=new_reference_duration,
            units_per_minute=new_units_per_minute,
            )
        return new_tempo

    def __truediv__(self, argument):
        r'''Divides tempo by `argument`. Operator required by Python 3.

        ..  container:: example

            **Example 1.** Divides tempo by number:

            ::

                >>> Tempo(Duration(1, 4), 60).__truediv__(2)
                Tempo(reference_duration=Duration(1, 4), units_per_minute=30)

        ..  container:: example

            **Example 2.** Divides tempo by other tempo:

            ::

                >>> Tempo(Duration(1, 4), 60).__truediv__(
                ...     Tempo(Duration(1, 4), 40)
                ...     )
                Multiplier(3, 2)

        Returns new tempo.
        '''
        return self.__div__(argument)

    ### PRIVATE PROPERTIES ###

    @property
    def _dotted(self):
        return self.reference_duration.lilypond_duration_string

    @property
    def _equation(self):
        if self.reference_duration is None:
            return
        if isinstance(self.units_per_minute, tuple):
            string = '{}={}-{}'
            string = string.format(
                self._dotted,
                self.units_per_minute[0],
                self.units_per_minute[1],
                )
            return string
        elif isinstance(self.units_per_minute, (float, Fraction)):
            markup = Tempo.make_tempo_equation_markup(
                self.reference_duration,
                self.units_per_minute,
                )
            string = str(markup)
            return string
        string = '{}={}'
        string = string.format(self._dotted, self.units_per_minute)
        return string

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        text, equation = None, None
        if self.textual_indication is not None:
            text = self.textual_indication
            text = schemetools.Scheme.format_scheme_value(text)
        if (self.reference_duration is not None and
            self.units_per_minute is not None):
            equation = self._equation
        if self.custom_markup is not None:
            return r'\tempo {}'.format(self.custom_markup)
        elif text and equation:
            return r'\tempo {} {}'.format(text, equation)
        elif equation:
            return r'\tempo {}'.format(equation)
        elif text:
            return r'\tempo {}'.format(text)
        else:
            return r'\tempo \default'

    def _make_lhs_score_markup(self, reference_duration=None):
        from abjad.tools import scoretools
        reference_duration = reference_duration or self.reference_duration
        selection = scoretools.make_notes([0], [reference_duration])
        markup = durationtools.Duration._to_score_markup(selection)
        return markup

    def _to_markup(self):
        from abjad.tools import markuptools
        if self.custom_markup is not None:
            return self.custom_markup
        duration_log = int(math.log(self.reference_duration.denominator, 2))
        lhs = markuptools.Markup.note_by_number(
            duration_log,
            self.reference_duration.dot_count,
            1,
            )
        lhs = lhs.general_align('Y', Down).fontsize(-6)
        equals = markuptools.Markup('=')
        #right_space = markuptools.Markup.hspace(0.1)
        units = markuptools.Markup(self.units_per_minute)
        #rhs = left_space + equals + right_space + units
        rhs = equals + units
        #rhs = rhs.fontsize(3).upright()
        rhs = rhs.upright()
        markup = lhs + rhs
        return markup

    ### PUBLIC PROPERTIES ###

    @property
    def custom_markup(self):
        r'''Gets custom markup of tempo.

        ..  container:: example

            **Example 1.** With custom markup:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(
                ...     Duration(1, 4),
                ...     67.5,
                ...     )
                >>> markup = markup.with_color('red')
                >>> tempo = Tempo(
                ...     reference_duration=Duration(1, 4),
                ...     units_per_minute=67.5,
                ...     custom_markup=markup,
                ...     )
                >>> staff = Staff("c'4 d'4 e'4 f'4")
                >>> score = Score([staff])
                >>> attach(tempo, staff)
                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score <<
                    \new Staff {
                        \tempo \markup {
                        \with-color
                            #red
                            {
                                \scale
                                    #'(0.75 . 0.75)
                                    \score
                                        {
                                            \new Score \with {
                                                \override SpacingSpanner.spacing-increment = #0.5
                                                proportionalNotationDuration = ##f
                                            } <<
                                                \new RhythmicStaff \with {
                                                    \remove Time_signature_engraver
                                                    \remove Staff_symbol_engraver
                                                    \override Stem.direction = #up
                                                    \override Stem.length = #5
                                                    \override TupletBracket.bracket-visibility = ##t
                                                    \override TupletBracket.direction = #up
                                                    \override TupletBracket.padding = #1.25
                                                    \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                    \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                    tupletFullLength = ##t
                                                } {
                                                    c'4
                                                }
                                            >>
                                            \layout {
                                                indent = #0
                                                ragged-right = ##t
                                            }
                                        }
                                =
                                \general-align
                                    #Y
                                    #-0.5
                                    67.5
                            }
                        }
                        c'4
                        d'4
                        e'4
                        f'4
                    }
                >>

        Set to markup or none.

        Defaults to none.

        Returns markup or none.
        '''
        return self._custom_markup

    @property
    def default_scope(self):
        r'''Gets default scope of tempo.

        ..  container:: example

            **Example 1.** Fifty-two eighth notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 8), 52)
                >>> tempo.default_scope
                <class 'abjad.tools.scoretools.Score.Score'>

        ..  container:: example

            **Example 2.** Ninety quarter notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> tempo.default_scope
                <class 'abjad.tools.scoretools.Score.Score'>

        Returns score.
        '''
        return self._default_scope

    @property
    def is_imprecise(self):
        r'''Is true if tempo is entirely textual or if tempo's
        units_per_minute is a range. Otherwise false.

        ..  container:: example

            **Example 1.** Imprecise tempos:

            ::

                >>> Tempo(Duration(1, 4), 60).is_imprecise
                False
                >>> Tempo(4, 60, 'Langsam').is_imprecise
                False
                >>> Tempo(textual_indication='Langsam').is_imprecise
                True
                >>> Tempo(4, (35, 50), 'Langsam').is_imprecise
                True
                >>> Tempo(Duration(1, 4), (35, 50)).is_imprecise
                True

        ..  container:: example

            **Example 2.** Precise tempo:

            ::

                >>> Tempo(Duration(1, 4), 60).is_imprecise
                False

        Returns true or false.
        '''
        if self.reference_duration is not None:
            if self.units_per_minute is not None:
                if not isinstance(self.units_per_minute, tuple):
                    return False
        return True

    @property
    def quarters_per_minute(self):
        r'''Gets quarters per minute of tempo.

        ..  container:: example

            **Example 1.** Fifty-two eighth notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 8), 52)
                >>> tempo.quarters_per_minute
                Fraction(104, 1)

        ..  container:: example

            **Example 2.** Ninety quarter notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> tempo.quarters_per_minute
                Fraction(90, 1)

        Returns tuple when tempo `units_per_minute` is a range.

        Returns none when tempo is imprecise.

        Returns fraction otherwise.
        '''
        if self.is_imprecise:
            return None
        if isinstance(self.units_per_minute, tuple):
            low = durationtools.Duration(1, 4) / self.reference_duration * \
                self.units_per_minute[0]
            high = durationtools.Duration(1, 4) / self.reference_duration * \
                self.units_per_minute[1]
            return (low, high)
        result = durationtools.Duration(1, 4) / self.reference_duration * \
            self.units_per_minute
        return Fraction(result)

    @property
    def reference_duration(self):
        r'''Gets reference duration of tempo.

        ..  container:: example

            **Example 1.** Fifty-two eighth notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 8), 52)
                >>> tempo.reference_duration
                Duration(1, 8)

        ..  container:: example

            **Example 2.** Ninety quarter notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> tempo.reference_duration
                Duration(1, 4)

        Returns duration.
        '''
        return self._reference_duration

    @property
    def textual_indication(self):
        r'''Gets optional textual indication of tempo.

        ..  container:: example

            **Example 1.** Fifty-two eighth notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 8), 52)
                >>> tempo.textual_indication is None
                True

        ..  container:: example

            **Example 2.** Ninety quarter notes per minute:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> tempo.textual_indication is None
                True

        Returns string or none.
        '''
        return self._textual_indication

    @property
    def units_per_minute(self):
        r'''Gets units per minute of tempo.

        ..  container:: example

            **Example 1.** Integer-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> tempo.units_per_minute
                90

        ..  container:: example

            **Example 2.** Float-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), 90.1)
                >>> tempo.units_per_minute
                90.1



            **Example 3.** Rational-valued tempo:

            ::

                >>> tempo = Tempo(Duration(1, 4), Fraction(181, 2))
                >>> tempo.units_per_minute
                Fraction(181, 2)

        Set to number or none.

        Defaults to none

        Returns number or none.
        '''
        return self._units_per_minute

    ### PUBLIC METHODS ###

    def duration_to_milliseconds(self, duration):
        r'''Gets millisecond value of `duration` under a given tempo.

        ..  container:: example

            **Example 1.** One quarter lasts 1000 msec at quarter equals 60:

            ::

                >>> tempo = Tempo((1, 4), 60)
                >>> tempo.duration_to_milliseconds(Duration(1, 4))
                Duration(1000, 1)

        ..  container:: example

            **Example 1.** Dotted sixteenth lasts 1500 msec at quarter equals
            60:

            ::

                >>> tempo = Tempo((1, 4), 60)
                >>> tempo.duration_to_milliseconds(Duration(3, 8))
                Duration(1500, 1)

        Returns duration.
        '''
        duration = durationtools.Duration(duration)
        # TODO: rewrite formula without line breaks;
        #       use two or three temporary variables instead.
        whole_note_duration = 1000 \
            * durationtools.Multiplier(
                self.reference_duration.denominator,
                self.reference_duration.numerator,
                ) \
            * durationtools.Multiplier(
                60,
                self.units_per_minute,
                )
        return durationtools.Duration(duration * whole_note_duration)

    def list_related_tempos(
        self,
        maximum_numerator=None,
        maximum_denominator=None,
        integer_tempos_only=False,
        ):
        r'''Lists related tempos.

        ..  container:: example

            **Example 1.** Rewrites tempo ``4=58`` by ratios ``n:d`` such that
            ``1 <= n <= 8`` and ``1 <= d <= 8``.

            ::

                >>> tempo = Tempo(Duration(1, 4), 58)
                >>> pairs = tempo.list_related_tempos(
                ...     maximum_numerator=8,
                ...     maximum_denominator=8,
                ...     )

            ::

                >>> for tempo, ratio in pairs:
                ...     string = '{!s}\t{!s}'.format(tempo, ratio)
                ...     print(string)
                4=29        1:2
                4=33+1/7    4:7
                4=34+4/5    3:5
                4=36+1/4    5:8
                4=38+2/3    2:3
                4=41+3/7    5:7
                4=43+1/2    3:4
                4=46+2/5    4:5
                4=48+1/3    5:6
                4=49+5/7    6:7
                4=50+3/4    7:8
                4=58        1:1
                4=66+2/7    8:7
                4=67+2/3    7:6
                4=69+3/5    6:5
                4=72+1/2    5:4
                4=77+1/3    4:3
                4=81+1/5    7:5
                4=87        3:2
                4=92+4/5    8:5
                4=96+2/3    5:3
                4=101+1/2   7:4
                4=116       2:1

        ..  container:: example

            **Example 2.** Integer-valued tempos only:

            ::

                >>> tempo = Tempo(Duration(1, 4), 58)
                >>> pairs = tempo.list_related_tempos(
                ...     maximum_numerator=16,
                ...     maximum_denominator=16,
                ...     integer_tempos_only=True,
                ...     )

            ::

                >>> for tempo, ratio in pairs:
                ...     string = '{!s}\t{!s}'.format(tempo, ratio)
                ...     print(string)
                4=29	1:2
                4=58	1:1
                4=87	3:2
                4=116	2:1

        Constrains ratios such that ``1:2 <= n:d <= 2:1``.

        Returns list of tempo / ratio pairs.
        '''
        from abjad.tools import sequencetools
        allowable_numerators = range(1, maximum_numerator + 1)
        allowable_denominators = range(1, maximum_denominator + 1)
        numbers = [allowable_numerators, allowable_denominators]
        enumeration = sequencetools.Enumeration(numbers)
        pairs = enumeration.yield_outer_product()
        multipliers = [durationtools.Multiplier(_) for _ in pairs]
        multipliers = [
            _ for _ in multipliers
            if Fraction(1, 2) <= _ <= Fraction(2)
            ]
        multipliers.sort()
        multipliers = sequencetools.Sequence(multipliers).remove_repeats()
        pairs = []
        for multiplier in multipliers:
            new_units_per_minute = multiplier * self.units_per_minute
            if (integer_tempos_only and not
                mathtools.is_integer_equivalent_number(new_units_per_minute)):
                continue
            new_tempo = type(self)(
                reference_duration=self.reference_duration,
                units_per_minute=new_units_per_minute,
                )
            ratio = mathtools.Ratio(multiplier.pair)
            pair = (new_tempo, ratio)
            pairs.append(pair)
        return pairs

    @staticmethod
    def make_tempo_equation_markup(reference_duration, units_per_minute):
        r'''Makes tempo equation markup.

        ..  container:: example

            **Example 1.** Integer-valued tempo:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(Duration(1, 4), 90)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \general-align
                        #Y
                        #-0.5
                        90
                    }

        ..  container:: example

            **Example 2.** Float-valued tempo:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(Duration(1, 4), 90.1)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \general-align
                        #Y
                        #-0.5
                        90.1
                    }

        ..  container:: example

            **Example 3.** Rational-valued tempo:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(
                ...     Duration(1, 4),
                ...     Fraction(181, 2),
                ...     )
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \raise
                        #-0.5
                        {
                            90
                            \tiny
                                \fraction
                                    1
                                    2
                        }
                    }

        ..  container:: example

            **Example 4.** Reference duration expressed with ties:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(Duration(5, 16), 90)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'4 ~
                                        c'16
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \general-align
                        #Y
                        #-0.5
                        90
                    }

        ..  container:: example

            **Example 5.** Reference duration expressed as a tuplet:

            ::

                >>> markup = Tempo.make_tempo_equation_markup(Duration(1, 6), 90)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        \tweak edge-height #'(0.7 . 0)
                                        \times 2/3 {
                                            c'4
                                        }
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \general-align
                        #Y
                        #-0.5
                        90
                    }

        ..  container:: example

            **Example 6.** Reference duration passed in as explicit rhythm:

            ::

                >>> durations = [Duration(1, 16), Duration(3, 16), Duration(1, 16)]
                >>> selection = scoretools.make_notes([0], durations)
                >>> attach(Tie(), selection)
                >>> attach(Beam(), selection)
                >>> markup = Tempo.make_tempo_equation_markup(selection, 90)
                >>> show(markup) # doctest: +SKIP

            ..  doctest::

                >>> print(format(markup))
                \markup {
                    \scale
                        #'(0.75 . 0.75)
                        \score
                            {
                                \new Score \with {
                                    \override SpacingSpanner.spacing-increment = #0.5
                                    proportionalNotationDuration = ##f
                                } <<
                                    \new RhythmicStaff \with {
                                        \remove Time_signature_engraver
                                        \remove Staff_symbol_engraver
                                        \override Stem.direction = #up
                                        \override Stem.length = #5
                                        \override TupletBracket.bracket-visibility = ##t
                                        \override TupletBracket.direction = #up
                                        \override TupletBracket.padding = #1.25
                                        \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                        \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                        tupletFullLength = ##t
                                    } {
                                        c'16 ~ [
                                        c'8. ~
                                        c'16 ]
                                    }
                                >>
                                \layout {
                                    indent = #0
                                    ragged-right = ##t
                                }
                            }
                    =
                    \general-align
                        #Y
                        #-0.5
                        90
                    }

            Pass rhythms like this as Abjad selections.

        Returns markup.
        '''
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if isinstance(reference_duration, selectiontools.Selection):
            selection = reference_duration
        else:
            selection = scoretools.make_notes([0], [reference_duration])
        lhs_score_markup = durationtools.Duration._to_score_markup(selection)
        lhs_score_markup = lhs_score_markup.scale((0.75, 0.75))
        equal_markup = markuptools.Markup('=')
        if (isinstance(units_per_minute, Fraction) and
            not mathtools.is_integer_equivalent_number(units_per_minute)):
            rhs_markup = markuptools.Markup.make_improper_fraction_markup(
                units_per_minute)
            rhs_markup = rhs_markup.raise_(-0.5)
        else:
            rhs_markup = markuptools.Markup(units_per_minute)
            rhs_markup = rhs_markup.general_align('Y', -0.5)
        markup = lhs_score_markup + equal_markup + rhs_markup
        return markup

    def rewrite_duration(self, duration, new_tempo):
        r'''Rewrites `duration` under `new_tempo`.

        ..  container:: example

            **Example 1.** Consider the two tempo indicators below.

            ::

                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> new_tempo = Tempo(Duration(1, 4), 90)

            `tempo` specifies quarter equal to ``60``.

            `new_tempo` indication specifies quarter equal to ``90``.

            `new_tempo` is ``3/2`` times as fast as `tempo`:

            ::

                >>> new_tempo / tempo
                Multiplier(3, 2)

            Note that a triplet eighth note under `tempo` equals a regular
            eighth note under `new_tempo`:

            ::

                >>> tempo.rewrite_duration(Duration(1, 12), new_tempo)
                Duration(1, 8)

            And note that a regular eighth note under `tempo` equals a dotted
            sixteenth under `new_tempo`:

            ::

                >>> tempo.rewrite_duration(Duration(1, 8), new_tempo)
                Duration(3, 16)

        Given `duration` governed by this tempo returns new duration governed
        by `new_tempo`.

        Ensures that `duration` and new duration consume the same amount of
        time in seconds.

        Returns duration.
        '''
        tempo_ratio = new_tempo / self
        new_duration = tempo_ratio * duration
        return new_duration



# -*- coding: utf-8 -*-
import collections
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import select


class PrototypeSelectorCallback(AbjadValueObject):
    r'''Prototype selector callback.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Callbacks'

    __slots__ = (
        '_flatten',
        '_head',
        '_prototype',
        '_tail',
        '_trim',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        prototype=None,
        flatten=None,
        head=None,
        tail=None,
        trim=None,
        ):
        prototype = prototype or ()
        if isinstance(prototype, collections.Sequence):
            prototype = tuple(prototype)
            assert all(isinstance(x, type) for x in prototype)
        assert isinstance(prototype, (tuple, type))
        self._prototype = prototype
        if flatten is not None:
            flatten = bool(flatten)
        self._flatten = flatten
        if head is not None:
            head = bool(head)
        self._head = head
        if tail is not None:
            tail = bool(tail)
        self._tail = tail
        if trim is not None:
            if isinstance(trim, collections.Sequence):
                trim = tuple(trim)
                assert all(isinstance(x, type) for x in trim)
            assert isinstance(trim, (tuple, type))
        self._trim = trim

    ### SPECIAL METHODS ###

    def __call__(self, argument, rotation=None):
        r'''Iterates tuple `argument`.

        Returns tuple of selections.
        '''
        import abjad
        assert isinstance(argument, tuple), repr(argument)
        result = []
        prototype = self.prototype
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        for subexpr in argument:
            subresult = iterate(subexpr).by_class(prototype)
            subresult = select(subresult)
            if subresult:
                if self.trim:
                    subresult = self._trim_subresult(subresult, self.trim)
                if self.head is not None:
                    subresult = self._head_filter_subresult(
                        subresult,
                        self.head,
                        )
                if self.tail is not None:
                    subresult = self._tail_filter_subresult(
                        subresult,
                        self.tail,
                        )
                if self.flatten:
                    result.extend(subresult)
                else:
                    result.append(subresult)
        return tuple(result)

    ### PRIVATE METHODS ###

    @staticmethod
    def _head_filter_subresult(result, head):
        import abjad
        result_ = []
        for item in result:
            if isinstance(item, abjad.Component):
                logical_tie = abjad.inspect_(item).get_logical_tie()
                if head == (item is logical_tie.head):
                    result_.append(item)
                else:
                    pass
            elif isinstance(item, abjad.Selection):
                if not all(isinstance(_, abjad.Component) for _ in item):
                    raise NotImplementedError(item)
                selection = []
                for component in item:
                    logical_tie = abjad.inspect_(component).get_logical_tie()
                    if head == logical_tie.head:
                        selection.append(item)
                    else:
                        pass
                selection = abjad.select(selection)
                result_.append(selection)
            else:
                raise TypeError(item)
        assert isinstance(result_, list), repr(result_)
        return abjad.select(result_)

    @staticmethod
    def _tail_filter_subresult(result, tail):
        import abjad
        result_ = []
        for item in result:
            if isinstance(item, abjad.Component):
                logical_tie = abjad.inspect_(item).get_logical_tie()
                if tail == (item is logical_tie.tail):
                    result_.append(item)
                else:
                    pass
            elif isinstance(item, abjad.Selection):
                if not all(isinstance(_, abjad.Component) for _ in item):
                    raise NotImplementedError(item)
                selection = []
                for component in item:
                    logical_tie = abjad.inspect_(component).get_logical_tie()
                    if tail == logical_tie.tail:
                        selection.append(item)
                    else:
                        pass
                selection = abjad.select(selection)
                result_.append(selection)
            else:
                raise TypeError(item)
        assert isinstance(result_, list), repr(result_)
        return abjad.select(result_)

    @staticmethod
    def _trim_subresult(result, trim):
        import abjad
        result_ = []
        found_good_component = False
        for item in result:
            if isinstance(item, abjad.Component):
                if not isinstance(item, trim):
                    found_good_component = True
            elif isinstance(item, abjad.Selection):
                if not all(isinstance(_, abjad.Component) for _ in item):
                    raise NotImplementedError(item)
                selection = []
                for component in item:
                    if not isinstance(component, trim):
                        found_good_component = True
                    if found_good_component:
                        selection.append(component)
                item = abjad.select(selection)
            else:
                raise TypeError(item)
            if found_good_component:
                result_.append(item)
        result__ = []
        found_good_component = False
        for item in reversed(result_):
            if isinstance(item, abjad.Component):
                if not isinstance(item, trim):
                    found_good_component = True
            elif isinstance(item, abjad.Selection):
                if not all(isinstance(_, abjad.Component) for _ in item):
                    raise NotImplementedError(item)
                selection = []
                for component in reversed(item):
                    if not isinstance(component, trim):
                        found_good_component = True
                    if found_good_component:
                        selection.insert(0, component)
                item = abjad.select(selection)
            else:
                raise TypeError(item)
            if found_good_component:
                result__.insert(0, item)
        assert isinstance(result__, list), repr(result__)
        result = abjad.select(result__)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def flatten(self):
        r'''Is true if selector callback returns a single, rather than nested
        selection. Otherwise false.

        Returns true or false.
        '''
        return self._flatten

    @property
    def head(self):
        r'''Is true when selector returns logical tie heads only.

        Is false when selector returns logical tie nonheads only.

        Is none when selector does not test for logical tie part.

        Set to true, false or none.

        Defaults to none.

        Returns true, false or none.
        '''
        return self._head

    @property
    def prototype(self):
        r'''Gets prototype selector callback prototype.

        Returns tuple of classes.
        '''
        return self._prototype

    @property
    def tail(self):
        r'''Is true when selector returns logical tie tails only.

        Is false when selector returns logical tie nontails only.

        Is none when selector does not test for logical tie part.

        Set to true, false or none.

        Defaults to none.

        Returns true, false or none.
        '''
        return self._tail

    @property
    def trim(self):
        r'''Gets trim prototype.

        Returns tuple of classes.
        '''
        return self._trim
